{ -----------------------------------------------------------------  }
{ CpConnec
  Autor:              Peter Zwosta
  Homepage: http://home.t-online.de/home/Peter.Zwosta                }
{ -----------------------------------------------------------------  }
{ TConnections: Helper class to manage connections, used in 
                TCapi (cpHand.pas)
  TConnection : a connection in TConnections
  TB3DataQueue: a queue for outgoing B3 data blocks
  TB3Data     : a data block in a TB3DataQueue                       }
{ -------------------------------------------------------------------}
{ History:
  Jun 28, 1999 pz, Ewart Nijburg TConnections.Destroy, FConnections.Free
  Nov 21, 1998 pz - Connection.Tag and ConnectionByTag added.
  Nov 7, 1998 pz - SentBlocks added to B3-Datablocks
         Queue.Free added in TB3DataQueue.Destroy 
  Nov 1, 1998 pz - TConnections[i] changed. i isn't the
         ConnectionID any more, but the Index.
         Added method ConnectionById.                                }
{ -----------------------------------------------------------------  }
unit CpConnec;

interface { ******************************************************** }
uses SysUtils, Dialogs, Messages, Classes, Windows, CpMsg;

const
{ ConnectStates for TConnection.State ------------------------------ }
csDisconnected       = $0000;
csDConnectPending    = $0001;
csDConnected         = $0002;
csBConnectPending    = $0004;
csConnected          = $0008;
csBDisconnectPending = $0010;
csDDisconnectPending = $0020;
{ ConnectTypes  ---------------------------------------------------- }
ctNone               = $0000;
ctActive             = $0001;                       { self initiated }
ctPassive            = $0002;                { call from third party }
{ DisconnectTypes  ------------------------------------------------- }
dtNone               = $0000;
dtActiveDisconnect   = $0001;                       { self initiated }
dtPassiveDisconnect  = $0002;          { disconnect from third party }

type
// EConnectionIDError = class(Exception);
{ -------------------------------------------------------------------}
{ TB3Data
  used in TB3DataQueue to manage the data blocks to be sent          }
{ -------------------------------------------------------------------}
TB3Data = class(TObject)
  DataHandle : word;                        { identifies a datablock }
  DataLength : word;                       { length of the datablock }
  DataPtr    : pointer;                        { pointer to the data }
  Flags      : word;                { parameter Flags of DATA_B3_REQ }
  Sent      : boolean;                       { true, if already sent }
end;
{ -------------------------------------------------------------------}
{ TB3DataQueue
  The queue is used to send more than one REQs when sending
  DATA_B3 messages                                                   }
{ -------------------------------------------------------------------}
TB3DataQueue = class(TObject)
    Queue : TList;
    FDataHandle : word;     { counter used to set TB3Data.DataHandle }

    Function GetCount : integer;
    Function GetItem(aIndex : integer) : TB3Data;
    Function GetSentBlocks : integer;  
  public
    constructor Create;
    destructor  Destroy; override;
    Function    Add(aDataPtr : pointer;
                    aDataLength : word;
                    aFlags : word) : word;
    Procedure   Delete(aDataHandle : word);
    Procedure   Clear;

    Property SentBlocks : integer read GetSentBlocks;
    Property Count : integer read GetCount;
    Property Items[Index : integer] : TB3Data read GetItem; default;
end;
{ -------------------------------------------------------------------}
{ TConnection
  Represents a connection (one B channel)                            }
{ -------------------------------------------------------------------}
TConnection = class(TObject)
  FPLCI : dword;                { Phyical Link Connection Identifier }
  procedure SetPLCI(PLCI : dword);           { Is a function, as ... }
  public                           { ... we get controller out of it }
    ID    : integer;                         { internal ConnectionID }
    Tag   : dWord;                         { can be used by the app. }
    { ---------------------------------------------------------------}
    ConnectType : word;               { active or passive connection }
    DisconnectType : word;                { active or pass. disconn. }
    State : word;                         { status of the connection }
    Controller : dword;                             { set in SetPLCI }
    NCCI : dword;            { Network Control Connection Identifier }
    { ---------------------------------------------------------------}
    B3DataQueue : TB3DataQueue;               { see. TCapi.AddB3Data }
    InB3DataTransferAsync : boolean;          { transfer in progress }

    constructor Create;
    destructor Destroy; override;

    property PLCI : dword read FPLCI write SetPLCI;
end;
{ -------------------------------------------------------------------}
{ TConnections                                                       }
{ Helper class to manage connections (TConnection)                   }
{ -------------------------------------------------------------------}
TConnections = class(TObject)
    FConnections : TList;                     { list of TConnections }
    Function GetCount : integer;
    Function GetItem(aIndex : integer) : TConnection;
  public
    constructor Create;
    destructor Destroy; override;
    function  CreateConnection : TConnection;
    procedure FreeConnection(var aConnection : TConnection);
    function  Validate(var aConnection : TConnection) : boolean;
    function  ConnectionByController(aController : dword) : TConnection;
    function  ConnectionByPLCI(aPLCI : dword) : TConnection;
    function  ConnectionByNCCI(aNCCI : dword) : TConnection;
    function  ConnectionByID(aConnectionID : integer) : TConnection;
    function  ConnectionByTag(aTag : dword) : TConnection;

    Property Count : integer read GetCount;
    Property Items[Index : integer] : TConnection read GetItem; default;
end;

const
  coInvalidID         = 0;
  coInvalidController = 0;
  coInvalidPLCI       = 0;
  coInvalidNCCI       = 0;

implementation { *************************************************** }

{ -------------------------------------------------------------------}
{ TB3DataQueue.Create                                                }
{ -------------------------------------------------------------------}
constructor TB3DataQueue.Create();
begin
  inherited Create;
  Queue := TList.Create;
  FDataHandle := 0;
end;
{ -------------------------------------------------------------------}
{ TB3DataQueue.Destroy                                               }
{ -------------------------------------------------------------------}
destructor TB3DataQueue.destroy;
begin
  Clear;
  Queue.Free;
  inherited Destroy;
end;
{ -------------------------------------------------------------------}
{ TB3DataQueue.GetCount                                              }
{ -------------------------------------------------------------------}
Function TB3DataQueue.GetCount : integer;
begin
  GetCount := Queue.Count;
end;
{ -------------------------------------------------------------------}
{ TB3DataQueue.GetItem
  Get Item with Index (Index is not the DataHandle)                  }
{ -------------------------------------------------------------------}
Function TB3DataQueue.GetItem(aIndex : integer) : TB3Data;
begin
  GetItem := TB3Data(Queue[aIndex]);
end;
{ -------------------------------------------------------------------}
{ GetSetBlocks
  Get number of B3 data blocks that are sent but not yet confirmed.
  Confirmed blocks are deleted in the queue (see CpHand.pas, TCapi)  }
{ -------------------------------------------------------------------}
Function TB3DataQueue.GetSentBlocks : integer;
var i, SentBlocks  : integer;
begin
  SentBlocks := 0;
  For i := 0 to Queue.Count - 1 do
    if TB3Data(Queue[i]).Sent then inc(SentBlocks);
  GetSentBlocks := SentBlocks;
end;
{ -------------------------------------------------------------------}
{ TB3DataQueue.Add
  Adds data to the queue and returns the DataHandle.                 }
{ -------------------------------------------------------------------}
Function TB3DataQueue.Add(aDataPtr : pointer;
                          aDataLength : word;
                          aFlags : word) : word;
var tmpB3Data : TB3Data;
begin
  inc(FDataHandle);
  if FDataHandle = MaxWord then FDataHandle := 1;
  { allocate memory and copy data to the queue }
  tmpB3Data := TB3Data.Create;
  GetMem(tmpB3Data.DataPtr, aDataLength);
  Move(aDataPtr^, tmpB3Data.DataPtr^, aDataLength);
  { store Datalength and DataHandle }
  tmpB3Data.DataLength := aDataLength;
  tmpB3Data.DataHandle := FDataHandle;
  tmpB3Data.Flags := aFlags;
  tmpB3Data.Sent := false;
  Queue.Add(tmpB3Data);                      { Add data to the queue }
  { return DataHandle to the app. }
  Add := tmpB3Data.DataHandle;
end;
{ -------------------------------------------------------------------}
{ TB3DataQueue.Delete
  Deletes a datablock with a given DataHandle.
  This method is used wenn CAPI sends a DATA_B3_CONF                 }
{ -------------------------------------------------------------------}
Procedure TB3DataQueue.Delete(aDataHandle : word);
var tmpB3Data : TB3Data;
    i, index  : integer;
begin
  index := -1; { default: not found }
  { find entry with DataHandle }
  For i := 0 to Queue.Count -1 do begin
     if TB3Data(Queue[i]).DataHandle = aDataHandle then begin
       index := i;
       break;
     end;
  end;
  if index = -1 then exit;           { exit if DataHandle not found  }
  { free data area and delete entry from the queue }
  tmpB3Data := TB3Data(Queue[index]);
  FreeMem(tmpB3Data.DataPtr, tmpB3Data.DataLength);
  tmpB3Data.Free;
  Queue.Delete(Index);
end;
{ -------------------------------------------------------------------}
{ TB3DataQueue.Clear
  deletes all entries of the queue and frees the allocated memory    }
{ -------------------------------------------------------------------}
Procedure TB3DataQueue.Clear;
var tmpB3Data : TB3Data;
begin
  While Queue.Count > 0 do begin
    tmpB3Data := TB3Data(Queue[0]);
    FreeMem(tmpB3Data.DataPtr, tmpB3Data.DataLength);
    tmpB3Data.Free;
    Queue.Delete(0);
  end;
end;
{ -------------------------------------------------------------------}
{ TConnection.Create                                                 }
{ -------------------------------------------------------------------}
constructor TConnection.Create;
begin
  inherited Create;
  ID          := coInvalidID;
  Tag         := 0;
  State       := csDisconnected;
  ConnectType := ctNone;
  DisconnectType := dtNone;
  PLCI        := coInvalidPLCI;
  NCCI        := coInvalidNCCI;
  Controller  := coInvalidController;
  B3DataQueue := TB3DataQueue.Create;
  InB3DataTransferAsync := false;
end;
{ -------------------------------------------------------------------}
{ TConnection.SetPLCI                                                }
{ -------------------------------------------------------------------}
procedure TConnection.SetPLCI(PLCI : dword);
begin
  FPLCI := PLCI;
  Controller := PLCI and $000000FF;
end;
{ -------------------------------------------------------------------}
{ TConnection.Destroy                                                }
{ -------------------------------------------------------------------}
destructor TConnection.Destroy;
begin
  B3DataQueue.Free;
  inherited Destroy;
end;
{ -------------------------------------------------------------------}
{ TConnections.Create                                                }
{ -------------------------------------------------------------------}
constructor TConnections.Create();
begin
  inherited Create;
  FConnections := TList.Create;
end;
{ -------------------------------------------------------------------}
{ TConnections.CreateConnection
  creates a connection and returns a pointer to the connection       }
{ -------------------------------------------------------------------}
function TConnections.CreateConnection : TConnection;
var tmpConnection : TConnection;
    i  : integer;
    id : integer;
begin
   tmpConnection := TConnection.Create;
   { find new ID, the application can use the ID as reference to the
     connection }
   id := 0;
   for i := 0 to FConnections.Count - 1 do begin
     if TConnection(FConnections[i]).id > id
        then id := TConnection(FConnections[i]).id;
   end;
   id := id + 1;
   if id = maxInt then id := 1;
   { set ID and add connection to the list }
   tmpConnection.ID := id;
   FConnections.Add(tmpConnection);
   { return connection }
   CreateConnection := tmpConnection;
end;
{ -------------------------------------------------------------------}
{ TConnections.FreeConnection                                        }
{ -------------------------------------------------------------------}
procedure TConnections.FreeConnection(var aConnection : TConnection);
var i  : integer;
begin
  i := FConnections.IndexOf(aConnection);
  if (i >= 0) then begin
    aConnection.Free;
    aConnection := nil;
    FConnections.Delete(i);
  end;
end;
{ -------------------------------------------------------------------}
{ TConnections.Validate
  checks if a connection is still valid and sets aConnection to nil
  if its not.                                                        }
{ -------------------------------------------------------------------}
function  TConnections.Validate(var aConnection : TConnection) : boolean;
var i  : integer;
begin
  i := FConnections.IndexOf(aConnection);
  if (i < 0) then begin
    aConnection := nil;
    Validate := false;
  end else Validate := true;
end;
{ -------------------------------------------------------------------}
{ TConnections.ConnectionByController                                }
{ -------------------------------------------------------------------}
function TConnections.ConnectionByController(aController : dword) : TConnection;
var i  : integer;
begin
   for i := 0 to FConnections.Count - 1 do begin
     if TConnection(FConnections[i]).Controller = aController then begin
        ConnectionByController := TConnection(FConnections[i]);
        exit;
     end;
   end;
   ConnectionByController := nil;
end;
{ -------------------------------------------------------------------}
{ TConnections.ConnectionByPLCI                                      }
{ -------------------------------------------------------------------}
function TConnections.ConnectionByPLCI(aPLCI : dword) : TConnection;
var i  : integer;
begin
   for i := 0 to FConnections.Count - 1 do begin
     if TConnection(FConnections[i]).PLCI = aPLCI then begin
        ConnectionByPLCI := TConnection(FConnections[i]);
        exit;
     end;
   end;
   ConnectionByPLCI := nil;
end;
{ -------------------------------------------------------------------}
{ TConnections.ConnectionByNCCI                                      }
{ -------------------------------------------------------------------}
function TConnections.ConnectionByNCCI(aNCCI : dword) : TConnection;
var i  : integer;
begin
   for i := 0 to FConnections.Count - 1 do begin
     if TConnection(FConnections[i]).NCCI = aNCCI then begin
        ConnectionByNCCI := TConnection(FConnections[i]);
        exit;
     end;
   end;
   ConnectionByNCCI := nil;
end;
{ -------------------------------------------------------------------}
{ TConnections.ConnectionByID                                        }
{ -------------------------------------------------------------------}
function  TConnections.ConnectionByID(aConnectionID : integer) : TConnection;
var i : integer;
begin
  For i := 0 to FConnections.Count - 1 do begin
    if aConnectionId = TConnection(FConnections[i]).id then begin
      ConnectionByID := TConnection(FConnections[i]);
      exit;
    end;
  end;
  ConnectionByID := nil;
end;
{ -------------------------------------------------------------------}
{ TConnections.ConnectionByTag                                       }
{ -------------------------------------------------------------------}
function  TConnections.ConnectionByTag(aTag : dWord) : TConnection;
var i : integer;
begin
  For i := 0 to FConnections.Count - 1 do begin
    if aTag = TConnection(FConnections[i]).Tag then begin
      ConnectionByTag := TConnection(FConnections[i]);
      exit;
    end;
  end;
  ConnectionByTag := nil;
end;
{ -------------------------------------------------------------------}
{ TConnections.Destroy                                               }
{ -------------------------------------------------------------------}
destructor TConnections.Destroy;
var tmpConnection : TConnection;
begin
  { free connections }
  While FConnections.Count > 0 do begin
    tmpConnection := TConnection(FConnections[0]);
    tmpConnection.Free;
    FConnections.Delete(0);
  end;
  FConnections.Free;
  FConnections := nil;  
  inherited Destroy;
end;
{ -------------------------------------------------------------------}
{ TConnections.GetCount                                              }
{ -------------------------------------------------------------------}
Function TConnections.GetCount : integer;
begin
  GetCount := FConnections.Count;
end;
{ -------------------------------------------------------------------}
{ TConnections.GetItem                                               }
{ -------------------------------------------------------------------}
Function TConnections.GetItem(aIndex : integer) : TConnection;
begin
  GetItem := TConnection(FConnections[aIndex]);
end;
end. 

