{ -----------------------------------------------------------------  }
{ CpConst
  Author:              Peter Zwosta
  Homepage: http://home.t-online.de/home/Peter.Zwosta                }
{ -------------------------------------------------------------------}
{ History:
  Feb 16, 1999 pz - added const for Supported Services (Capi Part III)             
  Nov 21, 1998 pz, rb - added consts for DTMF support                }
{ -------------------------------------------------------------------}
unit CpConst;

interface

uses SysUtils, Windows, Classes;

const
  { -----------------------------------------------------------------
    Type of message
    the following values are used as SubCommands in a MessageHeader  }
  CAPI_REQ   = $80; { request from application to CAPI               }
  CAPI_CONF  = $81; { CAPI confirms a request                        }
  CAPI_IND   = $82; { CAPI indicates an incoming event               }
  CAPI_RESP  = $83; { application confirms reception if incoming e.  }

  { -----------------------------------------------------------------
    Command and SubCommand of MessageHeader
    see CAPI_PUT_MESSAGE und CAPI_GET_MESSAGE                        }
  LISTEN_REQ             = $0580;    { Command 0x05, SubCommand 0x80 }
  LISTEN_CONF            = $0581;
  CONNECT_REQ            = $0280;
  CONNECT_CONF           = $0281;
  CONNECT_IND            = $0282;
  CONNECT_RESP           = $0283;
  CONNECT_ACTIVE_IND     = $0382;
  CONNECT_ACTIVE_RESP    = $0383;
  CONNECT_B3_REQ         = $8280;
  CONNECT_B3_CONF        = $8281;
  CONNECT_B3_IND         = $8282;
  CONNECT_B3_RESP        = $8283;
  CONNECT_B3_ACTIVE_IND  = $8382;
  CONNECT_B3_ACTIVE_RESP = $8383;
  CONNECT_B3_T90_ACTIVE_IND  = $8882;
  CONNECT_B3_T90_ACTIVE_RESP = $8883;
  DATA_B3_REQ            = $8680;
  DATA_B3_CONF           = $8681;
  DATA_B3_IND            = $8682;
  DATA_B3_RESP           = $8683;
  DISCONNECT_B3_REQ      = $8480;
  DISCONNECT_B3_CONF     = $8481;
  DISCONNECT_B3_IND      = $8482;
  DISCONNECT_B3_RESP     = $8483;
  DISCONNECT_REQ         = $0480;
  DISCONNECT_CONF        = $0481;
  DISCONNECT_IND         = $0482;
  DISCONNECT_RESP        = $0483;
  INFO_REQ               = $0880;
  INFO_CONF              = $0881;
  INFO_IND               = $0882;
  INFO_RESP              = $0883;
  ALERT_REQ              = $0180;
  ALERT_CONF             = $0181;
  FACILITY_REQ           = $8080;
  FACILITY_CONF          = $8081;
  FACILITY_IND           = $8082;
  FACILITY_RESP          = $8083;
  RESET_B3_REQ           = $8780;
  RESET_B3_CONF          = $8781;
  RESET_B3_IND           = $8782;
  RESET_B3_RESP          = $8783;
  SELECT_B_PROTOCOL_REQ  = $4180;
  SELECT_B_PROTOCOL_CONF = $4181;

  { -----------------------------------------------------------------
   Initialization of CAPI }
   { max. nr. of B-channels }
   defMaxLogicalConnections : dword = 1;
   { max. nr. of unconfirmed B3-Datablocks, CAPI supports max. 7 }
   defMaxBDataBlocks        : dword = 2;
   { max. Blocksize for B3 Datablocks, CAPI supports max. 2048 }
   defMaxBDataLen           : dword = 128;

  { -----------------------------------------------------------------
   consts for use in informative CAPI functions
   z.B. var Manufacturer : string;
        SetLength(Manufacturer, slManufacturer);
        Capi_GetManufacturer(PChar(Manufacturer));           }

   slManufacturer = 64;   { szBuffer f. Capi_GetManufacturer }
   slSerialNr     = 8;    { szBuffer f. Capi_GetSerialNumber }

  { -----------------------------------------------------------------
   const for use in infoelements in CAPI messages  }

  { default value for Controller (i.e. in LISTEN_REQ) }
  ctrDefault           = 1;

  { Info-Mask for LISTEN_REQ ----------------------------------------}
  imaDefault           = $007F;  { = imMaskAll }
  imaCause             = $0001;
  imaDateTime          = $0002;
  imaDisplay           = $0004;
  imaUserUserInfo      = $0008;
  imaCallProgression   = $0010;
  imaFacility          = $0020;
  imaCharging          = $0040;
  imaMaskAll           = $007F;

  { CIP-Mask for LISTEN_REQ -----------------------------------------}
  cipDefault             = $1FFF03FE;  { sets all Bits }
  cipAnyMatch            = $00000001;  { in CONNECT_IND look for BC, HLC u. LLC }
  cipSpeech              = $00000002;  { speech }
  cipUnrestDigitalInfo   = $00000004;  { unrestricted digital information }
  cipRestDigitalInfo     = $00000008;  { restricted digital information }
  cip31kHzAudio          = $00000010;  { 3.1 kHz audio }
  cip70kHzAudio          = $00000020;  { 7.0 kHz audio }
  cipVideo               = $00000040;  { video }
  cipPacketMode          = $00000080;  { packet mode }
  cip56kBits             = $00000100;  { 56 kBit/s rate adaptation }
  cipUnrestDigitalInfoTA = $00000200;  { unrestricted digital information with tones/announcements }
                                       { 10..15 reserved }
  cipTelephony           = $00010000;  { telephony }
  cipFaxGroup3           = $00020000;  { fax group 2/3 }
  cipFaxGroup4Class1     = $00040000;  { fax group 4 class 1 }
  cipTeletexbm           = $00080000;  { Teletex service (basic and mixed), fax group 4 class 2 }
  cipTeletexbp           = $00100000;  { Teletex service (basic and processable) }
  cipTeletexb            = $00200000;  { Teletex service (basic) }
  cipVideotex            = $00400000;  { Videotex }
  cipTelex               = $00800000;  { Telex }
  cipX400                = $01000000;  { message handling systems according X.400 }
  cipX200                = $02000000;  { OSI applications according X.200 }
  cip7kHzTelephony       = $04000000;  { 7 kHz Telephony }
  cipF721First           = $08000000;  { Video Telephony F.721, first connection }
  cipF721Second          = $10000000;  { Video Telephony F.721, second connection }

  { CIP-Mask2 for LISTEN_REQ ----------------------------------------}
  cip2Default            = 0;          { reserved }

  { CIP Values for CONNECT_IND --------------------------------------}
  civNoProfile           = 0;          { no predefined profile }
  civSpeech              = 1;          { speech - i.e. ISDN telephone}
  civUnrestDigitalInfo   = 2;          { unrestricted digital information }
  civRestDigitalInfo     = 3;          { restricted digital information }
  civ31kHzAudio          = 4;          { 3.1 kHz audio, i.e. analog telephone }
  civ70kHzAudio          = 5;          { 7.0 kHz audio }
  civVideo               = 6;          { video }
  civPacketMode          = 7;          { packet mode }
  civ56kBits             = 8;          { 56 kBit/s rate adaptation }
  civUnrestDigitalInfoTA = 9;          { unrestricted digital information with tones/announcements }
                                       { 10..15 reserved }
  civTelephony           = 16;         { telephony }
  civFaxGroup3           = 17;         { fax group 2/3 }
  civFaxGroup4Class1     = 18;         { fax group 4 class 1 }
  civTeletexbm           = 19;         { Teletex service (basic and mixed), fax group 4 class 2 }
  civTeletexbp           = 20;         { Teletex service (basic and processable) }
  civTeletexb            = 21;         { Teletex service (basic) }
  civVideotex            = 22;         { Videotex }
  civTelex               = 23;         { Telex }
  civX400                = 24;         { message handling systems according X.400 }
  civX200                = 25;         { OSI applications according X.200 }
  civ7kHzTelephony       = 26;         { 7 kHz Telephony }
  civF721First           = 27;         { Video Telephony F.721, first connection }
  civF721Second          = 28;         { Video Telephony F.721, second connection }


  { Parameter Reject for CONNECT_RESP -------------------------------}
  rjeAcceptCall   = $0000;
  rjeIgnoreCall   = $0001;  { other devices can accept the call }
  rjeNormalClear  = $0002;  { hang up }
  rjeUserBusy     = $0003;  { busy }
  rjeIncompDest   = $0007;  { incompatible destination }
  rjeUndefined    = $FFFF;  { not Capi defined }

  { Parameter Reject for CONNECT_B3_RESP, Parameter Reject ----------}
  rjeB3AcceptCall   = $0000;     { other devices can accept the call }
  rjeB3NormalClear  = $0002;                               { hang up }

  { TypeOfNumber (i.e. for TCallingPartyNumber)
    ETS 300 102, 4.5.10, extension Bit = 0 (Capi - Calling Party
                                            Number - Byte 0     -----}
  tonDefault         = 0;   { Bit 5-7 unknown, Bit 1-4 unknown       }
  tonInternationalNumberMask = $10; { Bit 5-7 international number   }
  tonNationalNumberMask      = $20; { Bit 5-7 national number        }

  { ScreenIndicator (i.e. for TCallingPartyNumber)
    ETS 300 102, 4.5.10,                           ------------------}
  sidDefault         = $80; { Bit 7+6 presentation allowed,
                              Bit 1+2 user provided, not screened    }
  sidNoPresentation  = $A0; { Bit 7+6 presentation restricted,
                              Bit 1+2 user provided, not screened    }
  { TypeOfSubAddress (i.e. for TCallingPartySubAddress) -------------}
  tosDefault     = $80;
  { Contents for TCallingPartySubAddress ----------------------------}
  tscDefault : string[1]    = chr($50);

  { Facility-Selectors (i.e. TFacilityReq) ------------------------- }
  fsHandset              = 0;
  fsDTMF                 = 1;
  fsV42bisCompression    = 2;
  fsSupplementaryService = 3;

  { DTMF-Suppport  --------------------------------------------------}
  { DTMF-Facility-functions }
  cDTMFStartListen     = 1;
  cDTMFStopListen      = 2;
  cDTMFSendDigits      = 3;
  { defaults for Facility Request Parameter }
  cDTMFDefToneDuration = 40;
  cDTMFDefGapDuration  = 40;
  { DTMF Information in Facility Conf. Parameter }
  cDTMFInfoSuccess        = 0;
  cDTMFInfoInvalidDigit   = 1;
  cDTMFInfoUnknownRequest = 2;
  { support for Supplementary Services ------------------------------}
  { Supplementary Services functions }
  cSSGetSupportedServices  : word = 0;
  cSSListen                : word = 1;
  { etc.}

{ Helper functions --------------------------------------------------}
function  CapiBufferSize(aMaxLogicalConnections : dWord) : dWord;
function  CapiGetCommandString(aCommandWord : word) : Ansistring;
function  CapiGetInfoString(aCapiInfo : dWord) : Ansistring;
function  CapiGetCipValueString(aCipValue : word) : Ansistring;

implementation { *************************************************** }

{ -------------------------------------------------------------------}
{ CapiBufferSize Fr CAPI_REGISTER                                   }
{ -------------------------------------------------------------------}
function  CapiBufferSize(aMaxLogicalConnections : dWord) : dWord;
begin
  { see. CAPI 2.0 doc.}
  CapiBufferSize := 1024 + (1024 * aMaxLogicalConnections);
end;

{ -------------------------------------------------------------------}
{ CapiGetCommandString
  returns the message name as string for a given Command/SubCommand  }
{ -------------------------------------------------------------------}
function CapiGetCommandString(aCommandWord : word) : Ansistring;
 var ax : Ansistring;
 begin
   case aCommandWord of
    LISTEN_REQ          : ax :=  'LISTEN_REQ';
    LISTEN_CONF         : ax :=  'LISTEN_CONF';
    CONNECT_IND         : ax :=  'CONNECT_IND';
    CONNECT_RESP        : ax :=  'CONNECT_RESP';
    CONNECT_REQ         : ax :=  'CONNECT_REQ';
    CONNECT_CONF        : ax :=  'CONNECT_CONF';
    CONNECT_ACTIVE_IND  : ax :=  'CONNECT_ACTIVE_IND';
    CONNECT_ACTIVE_RESP : ax :=  'CONNECT_ACTIVE_RESP';
    CONNECT_B3_REQ      : ax :=  'CONNECT_B3_REQ';
    CONNECT_B3_CONF     : ax :=  'CONNECT_B3_CONF';
    CONNECT_B3_IND      : ax :=  'CONNECT_B3_IND';
    CONNECT_B3_RESP     : ax :=  'CONNECT_B3_RESP';
    CONNECT_B3_ACTIVE_IND      : ax :=  'CONNECT_B3_ACTIVE_IND';
    CONNECT_B3_ACTIVE_RESP     : ax :=  'CONNECT_B3_ACTIVE_RESP';
    CONNECT_B3_T90_ACTIVE_IND  : ax :=  'CONNECT_B3_T90_ACTVE_IND';
    CONNECT_B3_T90_ACTIVE_RESP : ax :=  'CONNECT_B3_T90_ACTVE_RESP';
    DATA_B3_REQ         : ax :=  'DATA_B3_REQ';
    DATA_B3_CONF        : ax :=  'DATA_B3_CONF';
    DATA_B3_IND         : ax :=  'DATA_B3_IND';
    DATA_B3_RESP        : ax :=  'DATA_B3_RESP';
    DISCONNECT_B3_REQ   : ax :=  'DISCONNECT_B3_REQ';
    DISCONNECT_B3_CONF  : ax :=  'DISCONNECT_B3_CONF';
    DISCONNECT_B3_IND   : ax :=  'DISCONNECT_B3_IND';
    DISCONNECT_B3_RESP  : ax :=  'DISCONNECT_B3_RESP';
    DISCONNECT_REQ      : ax :=  'DISCONNECT_REQ';
    DISCONNECT_CONF     : ax :=  'DISCONNECT_CONF';
    DISCONNECT_IND      : ax :=  'DISCONNECT_IND';
    DISCONNECT_RESP     : ax :=  'DISCONNECT_RESP';
    INFO_REQ            : ax :=  'INFO_REQ';
    INFO_CONF           : ax :=  'INFO_CONF';
    INFO_IND            : ax :=  'INFO_IND';
    INFO_RESP           : ax :=  'INFO_RESP';
    ALERT_REQ           : ax :=  'ALERT_REQ';
    ALERT_CONF          : ax :=  'ALERT_CONF';
    FACILITY_REQ        : ax :=  'FACILITY_REQ';
    FACILITY_CONF       : ax :=  'FACILITY_CONF';
    FACILITY_IND        : ax :=  'FACILITY_IND';
    FACILITY_RESP       : ax :=  'FACILITY_RESP';
    RESET_B3_REQ        : ax :=  'RESET_B3_REQ';
    RESET_B3_CONF       : ax :=  'RESET_B3_CONF';
    RESET_B3_IND        : ax :=  'RESET_B3_IND';
    RESET_B3_RESP       : ax :=  'RESET_B3_RESP';
    SELECT_B_PROTOCOL_REQ      : ax :=  'SELECT_B_PROTOCOL_REQ';
    SELECT_B_PROTOCOL_CONF     : ax :=  'SELECT_B_PROTOCOL_CONF';
   else ax := ''
   end;
   CapiGetCommandString := ax;
 end;
{ -------------------------------------------------------------------}
{ CapiGetInfoString
  returns the Info-Text (see i.e. LISTEN_CONF) for a given Info
  parameter.                                                         }
{ -------------------------------------------------------------------}
function  CapiGetInfoString(aCapiInfo : dWord) : Ansistring;
 var ax : Ansistring;
begin
  case aCapiInfo of
     $0000: ax := 'OK';
     { error in CAPI_REGISTER -------------------------------------- }
     $1001: ax:= 'too many applications';
     $1002: ax:= 'logical block size too small, must be at least 128 bytes';
     $1003: ax:= 'buffer exceeds 64 Kbytes';
     $1004: ax:= 'message buffer size too small, must be at least 1024 bytes';
     $1005: ax:= 'max. number of logical connections not supported';
     $1007: ax:= 'the message could not be accepted because of an internal busy condition';
     $1008: ax:= 'OS Resource error (e.g. no memory)';
     $1009: ax:= 'COMMON-ISDN-API not installed';
     $100A: ax:= 'Controller does not support external equipment';
     $100B: ax:= 'Controller does only support external equipment';

     { errors in Capi_PutMessage and Capi_GetMessage ----------------}
     $1101: ax:= 'illegal application number';
     $1102: ax:= 'illegal command or subcommand or message length less than 12 octets';
     $1103: ax:= 'the message could not be accepted because of a queue full condition.';
     $1104: ax:= 'queue is empty';
     $1105: ax:= 'queue overflow, a message was lost.';
     $1106: ax:= 'unknown notification parameter';
     $1107: ax:= 'the message could not be accepted because of an internal busy condition';
     $1108: ax:= 'OS Resource error (e.g. no memory)';
     $1109: ax:= 'COMMON-ISDN-API not installed';
     $110A: ax:= 'Controller does not support external equipment';
     $110B: ax:= 'Controller does only support external equipment';

     { error information concerning resource / coding problems }
     $2001: ax:= 'message not supported in current state';
     $2002: ax:= 'illegal Controller/PLCI/NCCI';
     $2003: ax:= 'no PLCI available';
     $2004: ax:= 'no NCCI available';
     $2005: ax:= 'no LISTEN-Resources available';
     $2006: ax:= 'no FAX resources available(protocol T.30)';
     $2007: ax:= 'illegal message parameter coding';

     { error information concerning requested services }
     $3001: ax:= 'B1 protocol not supported';
     $3002: ax:= 'B2 protocol not supported';
     $3003: ax:= 'B3 protocol not supported';
     $3004: ax:= 'B1 protocol parameter not supported';
     $3005: ax:= 'B2 protocol parameter not supported';
     $3006: ax:= 'B3 protocol parameter not supported';
     $3007: ax:= 'B protocol combination not supported';
     $3008: ax:= 'NCPI not supported';
     $3009: ax:= 'CIP Value unknown';
     $300A: ax:= 'flags not supported (reserved bits)';
     $300B: ax:= 'facility not supported';
     $300C: ax:= 'data length not supported by current protocol';
     $300D: ax:= 'reset procedure not supported by current protocol';
     $300E: ax:= 'TEI assignment failed';

     { information about the clearing of a physical connection }
     $3301: ax:= 'Protocol error layer 1 (broken line or B-channel removed by signalling protocol)';
     $3302: ax:= 'Protocol error layer 2';
     $3303: ax:= 'Protocol error layer 3';
     $3304: ax:= 'Another application got that call';

     { T.30 specific reasons }
     $3311: ax := 'Connecting not successful (remote station is no FAX G3 machine)';
     $3312: ax := 'Connecting not successful (training error)';
     $3313: ax := 'Disconnected before transfer (remote station does not support transfer mode, e.g. resolution)';
     $3314: ax := 'Disconnected during transfer (remote abort)';
     $3315: ax := 'Disconnected during transfer (remote procedure error, e.g. unsuccessful repetition of T.30 commands)';
     $3316: ax := 'Disconnected during transfer (local tx data underrun)';
     $3317: ax := 'Disconnected during transfer (local rx data overflow)';
     $3318: ax := 'Disconnected during transfer (local abort)';
     $3319: ax := 'Illegal parameter coding (e.g. SFF coding error)';

     { disconnect causes from the network according to ETS 300 102-1/
       Q.931:
       34xx: In the field 'xx' the cause value received within a cause
       information element (octet 4) from the network is indicated
       (see ETS 300 102-1, 4.5.12 Cause)

       example: Cause number 2 'No route to specified transit network'
       --------------------------------------------------------------       
                fixed  class bits  value bits
       bits:        8     7  6  5     4 3 2 1
       --------------------------------------------------------------
                    1     0  0  0     0 0 1 0
       --------------------------------------------------------------
       -> Bits 1-8:          0100 0001 (4 value, 4 class)
       -> Class bits first:  0001 0100 (4 class, 4 value)
       -> Hex:                 $8   &2 ($3482)
     }
     $3481: ax := 'Unallocated (unassigned) number';
     $3482: ax := 'No route to specified transit network';
     $3483: ax := 'No route to destination';
     $3486: ax := 'Channel unacceptable';
     $3487: ax := 'Call awarded and being delivered in an established channel';
     $3490: ax := 'Normal call clearing';
     $3491: ax := 'User busy';
     $3492: ax := 'No user responding';
     $3493: ax := 'No answer from user (user alerted)';
     $3495: ax := 'Call rejected';
     $3496: ax := 'Number changed';
     $349A: ax := 'Non-selected user clearing';
     $349B: ax := 'Destination out of order';
     $349C: ax := 'Invalid number format';
     $349D: ax := 'Facility rejected';
     $349E: ax := 'Response to STATUS ENQUIRY';
     $349F: ax := 'Normal, unspecified';
     $34A2: ax := 'No circuit / channel available';
     $34A6: ax := 'Network out of order';
     $34A9: ax := 'Temporary failure';
     $34AA: ax := 'Switching equipment congestion';
     $34AB: ax := 'Access information discarded';
     $34AC: ax := 'Requested circuit / channel not available';
     $34AF: ax := 'Resources unavailable, unspecified';
     $34B1: ax := 'Quality of service unavailable';
     $34B2: ax := 'Requested facility not subscribed';
     $34B9: ax := 'Bearer capability not authorized';
     $34BA: ax := 'Bearer capability not presently available';
     $34BF: ax := 'Service or option not available, unspecified';
     $34C1: ax := 'Bearer capability not implemented';
     $34C2: ax := 'Channel type not implemented';
     $34C5: ax := 'Requested facility not implemented';
     $34C6: ax := 'Only restricted digital information bearer capability is available';
     $34CF: ax := 'Service or option not implemented, unspecified';
     $34D1: ax := 'Invalid call reference value';
     $34D2: ax := 'Identified channel does not exist';
     $34D3: ax := 'A suspended call exists, but this call identity does not';
     $34D4: ax := 'Call identity in use';
     $34D5: ax := 'No call suspended';
     $34D6: ax := 'Call having the requested call identity has been cleared';
     $34D8: ax := 'Incompatible destination';
     $34DB: ax := 'Invalid transit network selection';
     $34DF: ax := 'Invalid message, unspecified';
     $34E0: ax := 'Mandatory information element is missing';
     $34E1: ax := 'Message type non-existent or not implemented';
     $34E2: ax := 'Message not compatible with call state or message type non-existent or not implemented';
     $34E3: ax := 'Information element non-existent or not implemented';
     $34E4: ax := 'Invalid information element contents';
     $34E5: ax := 'Message not compatible with call state';
     $34E6: ax := 'Recovery on timer expiry';
     $34EF: ax := 'Protocol error, unspecified';
     $34FF: ax := 'Interworking, unspecified';
  else ax := 'unknown error'
  end;
  if aCapiInfo <> 0
    then CapiGetInfoString := ax + ' ($' + IntToHex(aCapiInfo, 4) + ')'
    else CapiGetInfoString := ax;
end;
{ -------------------------------------------------------------------}
{ CapiGetCipValueString
  returns the text for the parameter CipValue of the CONNECT_IND
  message                                                            }
{ -------------------------------------------------------------------}
function CapiGetCipValueString(aCipValue : word) : Ansistring;
var s : Ansistring;
begin
  s := '';
  case aCipValue of
    civNoProfile           : s := 'no predefined profile';
    civSpeech              : s := 'speech';
    civUnrestDigitalInfo   : s := 'unrestricted digital information';
    civRestDigitalInfo     : s := 'restricted digital information';
    civ31kHzAudio          : s := '3.1 kHz audio';
    civ70kHzAudio          : s := '7.0 kHz audio';
    civVideo               : s := 'video';
    civPacketMode          : s := 'packet mode';
    civ56kBits             : s := '56 kBit/s rate adaptation';
    civUnrestDigitalInfoTA : s := 'unrestricted digital information with tones/announcements';
    civTelephony           : s := 'telephony';
    civFaxGroup3           : s := 'fax group 2/3';
    civFaxGroup4Class1     : s := 'fax group 4 class 1';
    civTeletexbm           : s := 'Teletex service (basic and mixed), fax group 4 class 2';
    civTeletexbp           : s := 'Teletex service (basic and processable)';
    civTeletexb            : s := 'Teletex service (basic)';
    civVideotex            : s := 'Videotex';
    civTelex               : s := 'Telex';
    civX400                : s := 'message handling systems according X.400';
    civX200                : s := 'OSI applications according X.200';
    civ7kHzTelephony       : s := '7 kHz Telephony';
    civF721First           : s := 'Video Telephony F.721, first connection';
    civF721Second          : s := 'Video Telephony F.721, second connection';
  end;
  CapiGetCipValueString := s;
end;
end.
