{ -----------------------------------------------------------------  }
{ CpDLL
  Author:              Peter Zwosta
  Homepage: http://home.t-online.de/home/Peter.Zwosta                }
{ -----------------------------------------------------------------  }
{ Contains functions to load CAPI2032.DLL dynamically and contains
  the function declarations for the use of the functions provided
  by CAPI2032.DLL.
  In the finalization area the DLL is released automatically.        }
{ -------------------------------------------------------------------}
{ History:
{ Nov 1, 1998 pz - added CapiRealeaseDLL for Reset                   }
{ -----------------------------------------------------------------  }
unit CpDLL;

interface

uses
  Windows, SysUtils, Dialogs, Messages;

type
 { ---------------------------------------------------------
   CAPI function prototypes                                          }

 TCapi_Installed       = function  : DWord; StdCall;
 TCapi_GetManufacturer = procedure (SzBuffer : pChar); StdCall;
 TCapi_GetSerialNumber = function (SzBuffer : pChar) : dWord; StdCall;
 TCapi_GetVersion      = function (var CapiMajor, CapiMinor, ManufacturerMajor, ManufacturerMinor : dWord) : DWord; StdCall;
 TCapi_GetProfile      = function (pProfile : pointer; CtrlNr : DWord) : DWord; StdCall;

 TCapi_Register        = function (MsgBufferSize, MaxLogConn, MaxBDataBlocks,
                                   MaxBDataLen : DWord; pApplId : Pointer) : DWord; StdCall;
 TCapi_Release         = function (ApplId : DWord) : DWord; StdCall;
 TCapi_PutMessage      = function (ApplId : DWord; pCapiMessage : pointer) : DWord; StdCall;
 TCapi_GetMessage      = function (ApplId : DWord; var pCapiMessage : pointer) : DWord; StdCall;
 TCapi_WaitForSignal   = function (ApplId : DWord) : DWord; StdCall;


const { consts for CapiInitStatus - see CapiInitDLL  }
  ciOK                 = 0;
  ciNotInitialized     = 1;
  ciCapiDLLNotFound    = 2;
  ciCapiNotInstalled   = 3;

procedure CapiInitDLL;
function  CapiReleaseDLL : boolean;
function  CapiGetInitStatus : dWord;
function  CapiGetInitStatusString(aInitStatus : dWord) : Ansistring;

var { CAPi functions }
 { info functions of CAPI }
 Capi_Installed         : TCapi_Installed;
 Capi_GetManufacturer   : TCapi_GetManufacturer;
 Capi_GetSerialNumber   : TCapi_GetSerialNumber;
 Capi_GetVersion        : TCapi_GetVersion;
 Capi_GetProfile        : TCapi_GetProfile;
 { functions for the message exchange between app. and CAPI }
 Capi_Register          : TCapi_Register;
 Capi_Release           : TCapi_Release;
 Capi_PutMessage        : TCapi_PutMessage;
 Capi_GetMessage        : TCapi_GetMessage;
 Capi_WaitForSignal     : TCapi_WaitForSignal;

implementation { ****************************************************}

var  CapiLibHandle   : THandle;             { handle to CAPI2032.DLL }
     bCapiDllLoaded  : boolean;
     CapiInitStatus  : dWord;
type EDLLLoadError = class(Exception);
const
     CAPI_2032     = 'CAPI2032.DLL';

{ -------------------------------------------------------------------}
{ CapiInitDLL
  get the function pointers to the function provided by CAPI2032.DLL.
  On error CapiInitStatus is set to a value other than ciOK.
  CapiGetInitStatus can be used to retrieve the error.               }
{ -------------------------------------------------------------------}
procedure CapiInitDLL;
begin
  if CapiInitStatus = ciOK then exit;
  try
    CapiInitStatus  := ciOK;
    bCapiDLLLoaded  := true;
    CapiLibHandle   := LoadLibrary(CAPI_2032);
    if CapiLibHandle = 0 then
       raise EDLLLoadError.Create('File "CAPI2032.DLL" not found.');
    @Capi_Installed := GetProcAddress(CapiLibHandle,'CAPI_INSTALLED');
    if Capi_Installed <> 0 then
      begin
       CapiInitStatus := ciCapiNotInstalled;
       exit;
      end;
    @Capi_GetManufacturer := GetProcAddress(CapiLibHandle,'CAPI_GET_MANUFACTURER');
    @Capi_GetSerialNumber := GetProcAddress(CapiLibHandle,'CAPI_GET_SERIAL_NUMBER');
    @Capi_GetVersion      := GetProcAddress(CapiLibHandle,'CAPI_GET_VERSION');
    @Capi_GetProfile      := GetProcAddress(CapiLibHandle,'CAPI_GET_PROFILE');
    @Capi_Register        := GetProcAddress(CapiLibHandle,'CAPI_REGISTER');
    @Capi_Release         := GetProcAddress(CapiLibHandle,'CAPI_RELEASE');
    @Capi_PutMessage      := GetProcAddress(CapiLibHandle,'CAPI_PUT_MESSAGE');
    @Capi_GetMessage      := GetProcAddress(CapiLibHandle,'CAPI_GET_MESSAGE');
    @Capi_WaitForSignal   := GetProcAddress(CapiLibHandle,'CAPI_WAIT_FOR_SIGNAL');
  except
    On E : EDLLLoadError do begin
         CapiInitStatus := ciCapiDLLNotFound;
         bCapiDLLLoaded  := false;
      end;
  end;
end;
{ -------------------------------------------------------------------}
{ CapiReleaseDLL
  Releases the CAPI2032.DLL                                          }
{ -------------------------------------------------------------------}
function CapiReleaseDLL : boolean;
begin
  { Release handle to DLL }
  if bCapiDLLLoaded then begin
     if FreeLibrary(CapiLibHandle) then begin
        CapiInitStatus  := ciNotInitialized;
        bCapiDLLLoaded  := false;
        CapiReleaseDLL  := true;
        exit;
     end;
  end;
  CapiReleaseDLL := false;
end;
{ -------------------------------------------------------------------}
{ CapiGetInitStatus
  returns status of DLL initialization                               }
{ -------------------------------------------------------------------}
function  CapiGetInitStatus : dWord;
begin
  CapiGetInitStatus := CapiInitStatus;
end;
{ -------------------------------------------------------------------}
{ CapiGetInitStatusString
  returns corresponding error string                                 }
{ -------------------------------------------------------------------}
function CapiGetInitStatusString(aInitStatus : dWord) : Ansistring;
 var aStr : Ansistring;
begin
  case aInitStatus of
     ciNotInitialized    : aStr := 'CAPI2032.DLL not loaded.';
     ciOK                : aStr := 'CAPI 2.0 (32 Bit) installed';
     ciCapiDLLNotFound   : aStr := 'File "CAPI2032.DLL" not found.';
     ciCapiNotInstalled  : aStr := 'CAPI 2.0 (32 Bit) not installed.'
  else aStr := ''
  end;
  CapiGetInitStatusString := aStr;
end;

initialization { ****************************************************}
  CapiInitStatus  := ciNotInitialized;
  bCapiDLLLoaded  := false;
finalization { ******************************************************}
  CapiReleaseDLL;
end.
