{ -------------------------------------------------------------------}
{ CpHand
  Author:              Peter Zwosta
  Homepage: http://home.t-online.de/home/Peter.Zwosta                }
{ -----------------------------------------------------------------  }
{ CpHand contains the thread waiting for capi messages by using the
  CAPI_WAIT_FOR_SIGNAL API (TCapiCallBack) and the handler for CAPI
  messages                                                           }
{ -------------------------------------------------------------------}
{ History:
  Nov 20, 1998 pz - RESPs where the application has no alternatives
        are now sent by default before sending the IND to the
        application. ( CONNECT_AVTICE_RESP, CONNECT_B3_ACTIVE_RESP,
        DISCONNECT_B3_IND, DISCONNECT_IND, INFO_IND ...)
        Thanks to Rdiger Bund for his good ideas.
  Nov 7, 1998 pz - TransferB3DataAsync changed. OnGetB3Data
        (previously OnB3DataQueueEmpty) isn't called when all
        sent B3 datablocks are confirmed by CAPI, but always when
        any B3 datablock is confirmed and B3-Dataqueue doesn't
        contain any more blocks.
  Nov 1, 1998 pz - added Reset methode to enable applications to
        perform a CAPI reset automatically. For this purpose it was
        necessary to enable TCapi to receive messages (PostMessage,
        see TCapiCallback)                                           }
{ -------------------------------------------------------------------}
unit CpHand;

interface { ******************************************************** }
uses SysUtils, Dialogs, Messages, Classes, Windows,
     CpDLL, CpConnec, CpMsg, CpStruct, CpConst;

type  { -------------------------------------------------------------
        message type for sending CAPI messages from TCapiCallBack
        to TCapi }
      TCapiGetMessageFunc = procedure of object;
      { message type to send messages from TCapi to the application }
      TCapiEvent =
        procedure (aConnection : TConnection;
                   aMessagePtr : TCapiMessage;
                   var aDefaultHandling : boolean) of object;
      { used to send B3-datablocks asnychronously }
      TCapiGetB3Data =
        procedure (aConnection : TConnection;
                   var aStopTransfer : boolean;
                   var aSuccess : boolean) of object;
      TCapiAsyncTransferComplete = procedure(aConnection : TConnection;
                                        aSuccess : boolean) of object;
{ -------------------------------------------------------------------}
{ TCapiCallBack
  - calls CAPI_WAIT_FOR_SIGNAL and passes messages to TCapi by calling
    CapiGetMessageFunc
  - the message is saved in the property CapiMessage and has to be
    freed after use by TCapi                                         }
{ -------------------------------------------------------------------}
TCapiCallBack = class(TThread)
  FApplId             : dWord;
  FCapiGetMessageFunc : TCapiGetMessageFunc;
  FCapiMessage        : pointer;
  FWindowHandle       : HWND;            { to post messages to TCapi }
  protected
    procedure Execute; override;
  public
    Synchronizing      : boolean;      { Synchronize is being called }
    ConnectionsCleared : boolean;        { last connection is broken }
    constructor Create(ApplId : dWord;
                       aWindowHandle : HWND;
                       aCapiGetMessageFunc : TCapiGetMessageFunc);
    property CapiMessage : pointer read FCapiMessage;
end;

{ -------------------------------------------------------------------}
{ TCapi                                                              }
{ -------------------------------------------------------------------}
TCapi = class(TObject)
  private
    FApplId             : dWord;        { in CAPI-REGISTER from CAPI }
    FCapiCallBack       : TCapiCallBack;{ thread for WAIT_FOR_SIGNAL }
    FConnections        : TConnections;           { see CpConnec.pas }
    FWindowHandle       : HWND;                    { for PostMessage }

    { status flags }
    FIsApplRegistered   : boolean;                 { see CapiRelease }
    FError              : dWord;                       { error state }

    { values for CapiRegister}
    FMaxLogicalConnections : dWord;
    FMaxBDataBlocks        : dWord;
    FMaxBDataLen           : dWord;

    { messages to the application }
    FOnCapiEvent             : TCapiEvent;
    FOnGetB3Data             : TCapiGetB3Data;
    FOnAsyncTransferComplete : TCapiAsyncTransferComplete;
    FOnConnectionsCleared    : TNotifyEvent;

    Procedure CapiGetMessageFunc;
    Function  GetCapiError   : Ansistring;
    Function  GetErrorString : Ansistring;
    Procedure CapiRegister;
    Procedure CapiRelease;

    Procedure WndProc(var MsgRec: TMessage);
    Procedure HandleIndication(var aCapiMessage : TCapiMessage);
    Procedure HandleConfirmation(var aCapiMessage : TCapiMessage);
  public
    constructor Create(aMaxLogicalConnections : dWord;
                       aMaxBDataBlocks        : dWord;
                       aMaxBDataLen           : dWord);
    destructor  Destroy; override;
    function    Reset(aUnloadDLL : boolean) : boolean;

    Function PutMessage(aCapiMessage : TCapiMessage) : boolean;
    { functions to support easy connect and disconnect  }
    Function Connect(aConnectReq : TConnectReq) : TConnection;
    Function Disconnect(var aConnection : TConnection) : boolean;
    { functions to send B3-data }
    Function AddB3Data(var aConnection : TConnection;
                       aDataPtr : pointer;
                       aDataLength : word;
                       aFlags : word) : boolean;
    Function TransferB3DataAsync(var aConnection : TConnection) : boolean;
    Function TransferB3Data(var aConnection : TConnection;
                            aBlocksToSend : integer) : integer;
  published
    Property OnCapiEvent : TCapiEvent read FOnCapiEvent write FOnCapiEvent;
    Property OnGetB3Data : TCapiGetB3Data read FOnGetB3Data write FOnGetB3Data;
    Property OnAsyncTransferComplete : TCapiAsyncTransferComplete read  FOnAsyncTransferComplete write FOnAsyncTransferComplete;
    Property OnConnectionsCleared : TNotifyEvent read FOnConnectionsCleared write FOnConnectionsCleared;

    Property Connections : TConnections read FConnections;
    Property Error       : dword        read FError;
    Property CapiError   : Ansistring   read GetCapiError;
    Property ErrorString : Ansistring   read GetErrorString;
end;

implementation { *************************************************** }

{ the following functions make the event FOnConnectionsCleared
  possible }
Function  AllocateWindowHandle(aObject : TObject) : HWND; forward;
Procedure DeallocateWindowHandle(aWindowHandle : HWnd); forward;
Function  TCapiWindowProc( ahWnd   : HWND;
                           auMsg   : Integer;
                           awParam : WPARAM;
                           alParam : LPARAM): Integer; stdcall; forward;
const
  WM_CONNECTIONS_CLEARED = WM_USER + 20000;
  
  { error handling - see TCapi.GetErrorString}
  { erros from CapiDLL see CpDLL.pas, CapiGetInitStatusString }
  CapiDLLErrorBase        =  $9000;
  { errors from the Capi-parameter Info see CpConst.pas
    CapiGetInfoString }
  CapiInfoErrorBase       =  0;

{ -------------------------------------------------------------------}
{  TCapiCallBack.Create                                              }
{ -------------------------------------------------------------------}
constructor TCapiCallBack.Create(ApplId : dWord;
                                 aWindowHandle : HWND;
                                 aCapiGetMessageFunc : TCapiGetMessageFunc);
begin
  inherited Create(false);
  FApplId := ApplId;
  FWindowHandle := aWindowHandle;
  ConnectionsCleared := false;
  Synchronizing      := false;
  FCapiGetMessageFunc := aCapiGetMessageFunc;
end;
{ -------------------------------------------------------------------}
{  TCapiCallBack.Execute                                             }
{  The thread runs until it's terminated through a call to it's
   Terminate-method. Terminate sets the flag termined to true.
   On leaving the Execute-method the thread is terminated. The thread
   is not freed automatically (FreeOnTerminate <> true) but manually
   in TCapi.CapiRelease.
   As TCapiCallBack waits in Capi_WaitForSignal, TCapi has to send
   a dummy message to CAPI. TCapiCallBack leaves Capi_WaitForSignal
   to receive the CONF from CAPI and thus leaves Execute.            }
{ -------------------------------------------------------------------}
procedure TCapiCallBack.Execute;
  var dwRet : dWord;
begin
  while not terminated do begin
    { Capi_WaitForSignal is left when there's a message from CAPI    }
    dwRet := Capi_WaitForSignal(FApplId);
    if terminated then break;      { exit, if thread was terminated  }
    if (dwRet = 0) then begin
      { Get message from CAPI ... }
      dwRet := Capi_GetMessage(FApplId, FCapiMessage);
      { ... and pass it to TCapi  }
      { dwRet = 0 means there really is a message }
      if (dwRet = 0) then begin
        Synchronizing := true;
        Synchronize(FCapiGetMessageFunc);
        Synchronizing := false;
        if ConnectionsCleared and (FWindowHandle > 0) then begin
           ConnectionsCleared := false;
           { This PostMessage can cause that TCapiCallBack is
             terminated. Thus it is not possible to Synchronize for
             this purpose. }
           PostMessage(FWindowHandle, WM_CONNECTIONS_CLEARED, 0, 0);
        end;
      end;
    end;
  end; { while not terminated }
end;
{ -------------------------------------------------------------------}
{  TCapi.Create
   CAPI2032.DLL ist loaded, app. is registered with CAPI (-> FAppliD)
   and the thread waiting for CAPI messages (TCapiCallBack) is
   started.                                                          }
{ -------------------------------------------------------------------}
constructor TCapi.Create(aMaxLogicalConnections : dWord;
                         aMaxBDataBlocks : dWord;
                         aMaxBDataLen    : dWord);
begin
  inherited Create;
  FError            := 0;
  FOnCapiEvent      := nil;
  FOnGetB3Data             := nil;
  FOnAsyncTransferComplete := nil;
  FOnConnectionsCleared    := nil;

  FConnections      := TConnections.Create;
  FIsApplRegistered := false;

  FMaxLogicalConnections := aMaxLogicalConnections;
  FMaxBDataBlocks        := aMaxBDataBlocks;
  FMaxBDataLen           := aMaxBDataLen;

  FWindowHandle := AllocateWindowHandle(self);
  CapiRegister;                    { Load DLL and start CapiCallback }
end;
{ -------------------------------------------------------------------}
{  TCapi.Destroy                                                     }
{ -------------------------------------------------------------------}
Destructor TCapi.Destroy;
begin
  CapiRelease;               { unregister, terminate und free thread }
  FConnections.Free;                { free collection of connections }
  DeallocateWindowHandle(FWindowHandle);
  inherited Destroy;
end;
{ -------------------------------------------------------------------}
{ TCapi.CapiRegister
  Load DLL an register app. with CAPI                                }
{ -------------------------------------------------------------------}
Procedure TCapi.CapiRegister;
var wRes : dWord;
begin
  { Load DLL if not already loaded , see. CpDll.pas }
  if CapiGetInitStatus = ciNotInitialized then CapiInitDLL;
  if CapiGetInitStatus <> ciOK then begin
    FError := CapiDLLErrorBase + CapiGetInitStatus;
    exit;
  end;
  { register with CAPI -> CAPI assigns ApplID }
  wRes := Capi_Register(CapiBufferSize(FMaxLogicalConnections),
                        FMaxLogicalConnections,
                        FMaxBDataBlocks,
                        FMaxBDataLen, @FApplId);

  if wRes <> 0 then begin
    FError := CapiInfoErrorBase + wRes;
    exit;
  end;
  FIsApplRegistered := true;
  { start thread waiting for CAPI messages }
  FCapiCallback := TCapiCallback.Create(FApplId,
                                        FWindowHandle,
                                        CapiGetMessageFunc);
end;
{ -------------------------------------------------------------------}
{ TCapi.CapiRelease
  Unregisters app., terminates und frees thread CapiCallBack         }
{ -------------------------------------------------------------------}
Procedure TCapi.CapiRelease;
var wRes : dWord;
    i    : integer;
begin
  if FIsApplRegistered then begin            { if register succeeded }
    { Terminate sets the Flag Thread.terminated to true
      The thread stops if Thread.Execute ist finished }
    FCapiCallback.Terminate;
    { The CONF-message from CAPI dismisses the thread from
      Capi_WaitForSignal - as it's the same whether LISTEN_REQ
      succeeds or not the parameter controller is not important }
    PutMessage(
      TListenReq.Create( 1, imaDefault, cipSpeech, cip2Default,
         TCallingPartyNumber.CreateEmpty,
         TCallingPartySubAddress.CreateEmpty));

    { wait for termination of thread, then free thread object }
    WaitForSingleObject(FCapiCallback.Handle, 4000);
    FCapiCallBack.Free;

    wRes := Capi_Release(FApplID);           { unregister from CAPI  }

    if (wRes <> 0) then FError := CapiInfoErrorBase + wRes;
  end;

  { inform application about connections in transfer state }
  For i := 0 to FConnections.Count - 1 do begin
    if FConnections[i].InB3DataTransferAsync
      then if assigned(FOnAsyncTransferComplete)
             then FOnAsyncTransferComplete(FConnections[i], false);
  end;
end;
{ -------------------------------------------------------------------}
{ TCapi.Reset
  Unregisters the app. from CAPI and reregisters it. aUnloadDLL
  lets this function try to unload the CAPI2032.DLL.
  If CAPI is in use in other parts of the application, aUnloadDLL
  does not succeed. aUnloadDLL suceeds if another application uses
  CAPI2032.DLL.                                                      }
{ -------------------------------------------------------------------}
function  TCapi.Reset(aUnloadDll : boolean) : boolean;
begin
  { During a connection or while a message from CAPI is handled
    Reset is not allowed as TCapiCallback is calling Synchronize
    at that moment and thus cant be terminated. }
  if (assigned(FCapiCallBack) and FCapiCallback.Synchronizing) or
     (FConnections.Count > 0) then begin
    Reset := false;
    exit;
  end;
  try
    CapiRelease;  { unregister with CAPI and terminate FCapiCallback }
    if aUnloadDLL then CapiReleaseDLL;                  { unload DLL }
    CapiRegister;       { register with CAPI and start FCapiCallback }
    Reset := (FError = 0);
  except
    On exception do Reset := false;
  end;
end;
{ -------------------------------------------------------------------}
{  TCapi.AddB3Data
   Copies Data from the application to the B3 dataqueue of the
   connection. The application can free its data after calling
   AddB3Data. In TCapi.HandleConfirmation TCapi releases the memory
   itself, when the corresponding DATA_B3_CONF arrives.
   In TransferB3Data or TransferB3DataAsync the data is sent to CAPI.} 
{ -------------------------------------------------------------------}
Function TCapi.AddB3Data(var aConnection : TConnection;
                         aDataPtr : pointer;
                         aDataLength : word;
                         aFlags : word) : boolean;
begin
  if FConnections.Validate(aConnection) then begin
    aConnection.B3DataQueue.Add(aDataPtr, aDataLength, aFlags);
    AddB3Data := true;
  end else AddB3Data := false;
end;
{ -------------------------------------------------------------------}
{ TCapi.TransferB3DataAsync
  This method can be used to transfer data, that's fully available
  in advance. The app. provides two event handlers:
  - In FOnGetB3Data the app. adds data to the B3DataQueue (AddB3Data)
    After leaving the event handler the data in B3DataQueue is sent
    via TransferB3Data.
    When the corresponding DATA_B3_CONF arrives, memory for the data
    is freed automatically and TransferB3DataAsync is called again.
    If there are still datablocks in the B3DataQueue they are sent.
    If there aren't FOnGetB3Data is called to enable the app. to
    provide new datablocks. If the application doesn't provide a
    handler for FOnGetB3Data, only the blocks in B3DataQueue are sent
    and after that the transfer stops.
  - FOnAsyncTransferComplete informs the app. when a transfer is
    completed. When the app. starts a transfer it can rely on being
    informed about the end of the transfer. FOnAsyncTransferComplete
    is fired in the following occasions:
    - The app. sets the parameter StopTransfer in FOnGetB3Data to true
      to stop the transfer.
    - An unrecoverable error occured during transfer
    - The connection is broken
    - TCapi is freed and a connection is still in transfer mode.     }
{ -------------------------------------------------------------------}
Function TCapi.TransferB3DataAsync(var aConnection : TConnection) : boolean;
var SentBlocks, BlocksToSend   : integer;
    OK           : boolean;
    StopTransfer : boolean;
    Success      : boolean;
begin
  if (not FConnections.Validate(aConnection)) then begin
    TransferB3DataAsync := false;
    exit;
  end;
  { If this method is called at first, the connection is set
    in transfer mode }
  if (not aConnection.InB3DataTransferAsync)
    then aConnection.InB3DataTransferAsync := true;

  { The app. can use StopTransfer to stop the transfer and
    prevent future FOnGetB3Data events. }
  StopTransfer := false;
  { Success can be set by the app. via FOnGetB3Data to
    forward erros to FOnCapiAsyncTransferComplete }
  Success := true;

  { FOnGetB3Data is called when all datablocks in B3DataQueue are
    sent (they don't have to be confirmed) }
  if (aConnection.B3DataQueue.SentBlocks = aConnection.B3DataQueue.Count)
    then if assigned(FOnGetB3Data)
           then FOnGetB3Data(aConnection, StopTransfer, Success)
           else StopTransfer := true;

  { TransferB3Data sends the datablocks to CAPI.
    FMaxBDataBlocks is the max. number of blocks that are allowed
    to be sent without CONF. SentBlocks are the blocks already sent
    but not yet confirmed. }
  OK := true;
  if not StopTransfer then begin
    BlocksToSend := integer(FMaxBDataBlocks) - aConnection.B3DataQueue.SentBlocks;
    SentBlocks := TransferB3Data(aConnection, BlocksToSend);
    OK := (BlocksToSend = SentBlocks);
  end;

  { inform app. about end of transfer }
  if StopTransfer or (not OK) then begin
    aConnection.InB3DataTransferAsync := false;
    if assigned(FOnAsyncTransferComplete)
      then FOnAsyncTransferComplete(aConnection, OK and Success);
  end;

  { This method can return false only if FOnGetB3Data wasn't called
    yet }
  TransferB3DataAsync := true;
end;
{ -------------------------------------------------------------------}
{  TCapi.TransferB3Data
   Sends data from the DataQueue of a connection to CAPI
   !!! During transmission no CONF from CAPI is allowed to delete a
   queue entry.                                                      }
{ -------------------------------------------------------------------}
Function TCapi.TransferB3Data(var aConnection : TConnection;
                              aBlocksToSend : integer) : integer;
var tmpB3Data : TB3Data;
    OK : boolean;
    i, BlocksSent : integer;
begin
  if not FConnections.Validate(aConnection) then begin
    TransferB3Data := 0;
    exit;
  end;
  OK := true;                { will be false only if an error occurs }
  i  := 0;
  BlocksSent := 0;
  { The blocks are sent in the same order the app. provided them }
  While (BlocksSent < aBlocksToSend) and
        (i < aConnection.B3DataQueue.Count) do begin
    tmpB3Data := aConnection.B3DataQueue[i];
    if not tmpB3Data.Sent then begin
      OK := PutMessage(
              TDataB3Req.Create(
                aConnection.NCCI,
                dword(tmpB3Data.DataPtr),
                tmpB3Data.DataLength,
                tmpB3Data.DataHandle,
                tmpB3Data.Flags));
      if OK then begin
        tmpB3Data.Sent := true;
        inc(BlocksSent);
      end else break;
    end;
    inc(i);
  end;
  { If there's an error in PutMessage the app. has to repeat
    the last FOnGetB3Data }
  if not OK then aConnection.B3DataQueue.Clear;
  TransferB3Data := BlocksSent;
end;
{ -------------------------------------------------------------------}
{  TCapi.Connect
   Creates a new connection and returns it                           }
{ -------------------------------------------------------------------}
Function TCapi.Connect(aConnectReq : TConnectReq) : TConnection;
var Connection : TConnection;
begin
  { MessageNumber is used to recognize the CONNECT_REQ when the
    corrseponing CONNECT_CONF arrives.
    For other purposes the MessageNumber isn't used. }
   Connection := FConnections.CreateConnection;
   Connection.ConnectType    := ctActive;
   Connection.DisconnectType := dtNone;
   Connection.State          := csDisconnected;
   aConnectReq.Header.MessageNumber := Connection.Id;

   PutMessage(aConnectReq);
   Connect := Connection;
end;
{ -------------------------------------------------------------------}
{  TCapi.Disconnect
   If the app. doesn't want to use this method to hang up, it has to
   send DISCONNECT_REQ in DISCONNECT_B3_IND itself as
   ctActiveDisconnect is not set automatically.                      }
{ -------------------------------------------------------------------}
Function TCapi.Disconnect(var aConnection : TConnection) : boolean;
begin
  FError := 0;
  if not FConnections.Validate(aConnection) then begin
    Disconnect := false;
    exit;
  end;

  case aConnection.State of
   {-----------------------------------------------------------------}
   csConnected : begin
      aConnection.DisconnectType := dtActiveDisconnect;
      PutMessage(
        TDisconnectB3Req.Create(
            aConnection.NCCI,
            TStruct.CreateEmpty));
     end;
   {-----------------------------------------------------------------}
   csBConnectPending,
   csBDisconnectPending,
   csDConnected,
   csDConnectPending,
   csDDisconnectPending : begin
      aConnection.DisconnectType := dtActiveDisconnect;
      PutMessage(
        TDisconnectReq.Create(
            aConnection.PLCI,
            TStruct.CreateEmpty));
    end;
  end;
  Disconnect := (FError = 0);
end;
{ -------------------------------------------------------------------}
{  TCapi.CapiGetMessageFunc
   is called by TCapiCallBack to pass a new message from CAPI.
   CapiCreateMessage (see. CpMsg.pas) copies the data of the message
   in its own memory area.                                           }
{ -------------------------------------------------------------------}
Procedure TCapi.CapiGetMessageFunc;
var aCapiMessage  : TCapiMessage;
begin
  aCapiMessage := CapiCreateMessage(FCapiCallBack.CapiMessage);
  case aCapiMessage.Header.SubCommand of
    CAPI_IND  : HandleIndication(aCapiMessage);
    CAPI_CONF : HandleConfirmation(aCapiMessage);
  end;
end;
{ -------------------------------------------------------------------}
{  TCapi.PutMessage                                                  }
{  is called by the app. to send messages to CAPI                    }
{ -------------------------------------------------------------------}
Function TCapi.PutMessage(aCapiMessage : TCapiMessage) : boolean;
var wRes : dWord;
begin
  FError := 0;
  { ApplId (see CapiRegister) is the only field in the header that's
    provided here. TotalLength and WordCommand are set during
    creation of a message object (i.e. CpMsg.pas, TConnectReq.Create).
    Default MessageNumber is 0 (see CpMsg.pas TCapiMessage-
    Header.Create) }
  aCapiMessage.Header.ApplId := FApplID;

  { send to CAPI }
  wRes := Capi_PutMessage(FApplId, aCapiMessage.MessagePtr);
  if (wRes <> 0) then FError := CapiInfoErrorBase + wRes;
  CapiFreeMessage(aCapiMessage);

  PutMessage := (wRes = 0);                      { true, if wRes = 0 }
end;
{ -------------------------------------------------------------------}
{  TCapi.HandleIndication
{ -------------------------------------------------------------------}
Procedure TCapi.HandleIndication(var aCapiMessage : TCapiMessage);
var Connection  : TConnection;
    defHandling : boolean;
begin
  FError := 0;
  defHandling := true;
  Connection  := nil;              { default: no connection assigned }

  try
  try
  case aCapiMessage.Header.WordCommand of
    { ---------------------------------------------------------------}
    { After CONNECT_IND the app. can send an ALERT_REQ to let the
      user decide what to do in CONNECT_RESP. Without an ALERT_REQ
      the call is rejected by the network (no user responding)
      between 4 sec. }
    CONNECT_IND : begin
      Connection := FConnections.CreateConnection;
      Connection.ConnectType    := ctPassive;
      Connection.DisconnectType := dtNone;
      Connection.State          := csDConnectPending;
      Connection.PLCI           := TConnectInd(aCapiMessage).PLCI;
      { forward CONNECT_IND to the app. }
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    { if defhandling = true, B3_REQ is sent immediately with empty
      NCPI (only active connections) }
    CONNECT_ACTIVE_IND : begin
      Connection :=
        FConnections.ConnectionByPLCI(TConnectActiveInd(aCapiMessage).PLCI);
      Connection.State := csDConnected;
      { answer immediately - app. has no choice }
      PutMessage(TConnectActiveResp.Create(Connection.PLCI));
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
      { send B3_REQ when connection is active (we are the caller) }
      if defHandling and
         (Connection.ConnectType = ctActive) then begin
        PutMessage(
          TConnectB3Req.Create(
            Connection.PLCI,
            TStruct.CreateEmpty));
      end;
    end;
    { ---------------------------------------------------------------}
    { NCCI is provided by CAPI
      if defHandling = true, the call is accepted                    }
    CONNECT_B3_IND : begin
      Connection :=
            FConnections.ConnectionByPLCI(
                  TConnectB3Ind(aCapiMessage).NCCI and $0000FFFF);
      Connection.State := csBConnectPending;
      Connection.NCCI  := TConnectB3Ind(aCapiMessage).NCCI;
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
      if defHandling then
        PutMessage(
            TConnectB3Resp.Create(
              Connection.NCCI,
              rjeB3AcceptCall,
              TStruct.CreateEmpty));
    end;
    { ---------------------------------------------------------------}
    { When this message arrives a connection is complete (NCCI)      }
    CONNECT_B3_ACTIVE_IND : begin
      Connection :=
        FConnections.ConnectionByNCCI(TConnectB3ActiveInd(aCapiMessage).NCCI);
      Connection.State := csConnected;
      { send RESP before event so the app. can send data immediately }
      PutMessage(TConnectB3ActiveResp.Create(TConnectB3ActiveInd(aCapiMessage).NCCI));
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    { change from T.70 to T.90                                       }
    CONNECT_B3_T90_ACTIVE_IND : begin
      Connection :=
        FConnections.ConnectionByNCCI(TConnectB3T90ActiveInd(aCapiMessage).NCCI);
      PutMessage(TConnectB3T90ActiveResp.Create(TConnectB3T90ActiveInd(aCapiMessage).NCCI));
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    { The app. has to use data immediately as by default a
      DATA_B3_RESP is sent what means to CAPI it can free the memory
      area }
    DATA_B3_IND : begin
      Connection :=
        FConnections.ConnectionByNCCI(TDataB3Ind(aCapiMessage).NCCI);
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
      if defHandling then
        PutMessage(
          TDataB3Resp.Create(
            Connection.NCCI,
            TDataB3Ind(aCapiMessage).DataHandle));
    end;
    { ---------------------------------------------------------------}
    { DISCONNECT_B3_IND always arrives when a layer 3 connection
      is disconnected (no matter if active or passive connection).
      DISCONNECT_B3_IND is the last message from CAPI to an NCCI.    }
    DISCONNECT_B3_IND : begin
      Connection :=
        FConnections.ConnectionByNCCI(TDisconnectB3Ind(aCapiMessage).NCCI);
      Connection.State := csDDisconnectPending;
      if Connection.DisconnectType = dtNone
        then Connection.DisconnectType := dtPassiveDisconnect;
      PutMessage(TDisConnectB3Resp.Create(Connection.NCCI));
      if Assigned(FOnCapiEvent)
        then FOnCapiEvent(Connection, aCapiMessage, defHandling);
      { if the app. has caused the disconnect send DISCONNECT_REQ }
      if defHandling and
         (Connection.DisconnectType = dtActiveDisconnect) then begin
        PutMessage(
          TDisconnectReq.Create(
            Connection.PLCI,
            TStruct.CreateEmpty));
      end;
    end;
    { ---------------------------------------------------------------}
    { DISCONNECT_IND always is the last msg. from CAPI when a
      CONNECT_IND had occured -> the connection can be released      }
    DISCONNECT_IND : begin
      Connection :=
        FConnections.ConnectionByPLCI(TDisconnectInd(aCapiMessage).PLCI);
      if assigned(Connection) then begin
        Connection.State := csDisconnected;
        { Important !!! FError can be <> 0 without an error } 
        FError := CapiInfoErrorBase + TDisconnectInd(aCapiMessage).Reason;
        PutMessage(
          TDisconnectResp.Create(TDisconnectInd(aCapiMessage).PLCI));
        if assigned(FOnCapiEvent)
           then FOnCapiEvent(Connection, aCapiMessage, defHandling);
        { if a connection is released transfer mode has to be
          terminated }
        if (Connection.InB3DataTransferAsync)
           then if assigned(FOnAsyncTransferComplete)
                  then FOnAsyncTransferComplete(Connection, false);
        FConnections.FreeConnection(Connection);
        { FCapiCallBack sends FOnConnectionsCleared. Here we mustn't
          call FOnConnectionsCleared as TCapiCallBack is still in
          Synchronize and TCapi.Reset calls FCapiCallback.Free }
        if assigned(FOnConnectionsCleared) and (FConnections.Count = 0)
          then FCapiCallback.ConnectionsCleared := true;
      end else begin
        { This shouldn't happen but it does if the app. needs more
          than 4 or 5 sec. to handle DISCONNECT_IND }
        PutMessage(
         TDisconnectResp.Create(TDisconnectInd(aCapiMessage).PLCI));
      end;
    end;
    { ---------------------------------------------------------------}
    { ConnectionID can be nil if the info isn't related to a special
      PLCI }
    INFO_IND : begin
      Connection := FConnections.ConnectionByPLCI(TInfoInd(aCapiMessage).PLCI);
      PutMessage(
        TInfoResp.Create(TInfoInd(aCapiMessage).PLCI));
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    FACILITY_IND : begin
      PutMessage(
         TFacilityResp.Create(
            TFacilityInd(aCapiMessage).Controller_PLCI_NCCI,
            TFacilityInd(aCapiMessage).FacilitySelector,
            TStruct.CreateEmpty));
      if Assigned(FOnCapiEvent)
          then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    RESET_B3_IND : begin
      Connection :=
        FConnections.ConnectionByNCCI(TResetB3Ind(aCapiMessage).NCCI);
      PutMessage(
        TResetB3Resp.Create(TResetB3Ind(aCapiMessage).NCCI));
      if Assigned(FOnCapiEvent)
          then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
  end; { case aCapiMessage.Header.WordCommand }
  except on exception do ;
  end; { try }
  finally
   CapiFreeMessage(aCapiMessage);
  end;
end;
{ -------------------------------------------------------------------}
{  TCapi.HandleConfirmation
   Errors <= $00FF in _CONF-Messages are warnings. Nevertheless
   the REQ was sent succesfully                                      }
{ -------------------------------------------------------------------}
Procedure TCapi.HandleConfirmation(var aCapiMessage : TCapiMessage);
var Connection  : TConnection;
    ConnectionId : integer;
    defHandling : boolean;
    ConfInfo    : word;
    ConfError   : boolean;
begin
  FError := 0;
  defHandling := true;
  Connection := nil;               { default: no connection assigned }

  { DATA_B3_CONF has DataHandle in the second field and not Info }
  if aCapiMessage.Header.WordCommand = DATA_B3_CONF
    then ConfInfo := TDataB3Conf(aCapiMessage).Info
    else ConfInfo := TCapiConfirmation(aCapiMessage).Info;

  ConfError := (ConfInfo > $00FF);            { <= 00FF are warnings }
  if ConfInfo > $00FF then FError := CapiInfoErrorBase + ConfInfo;

  case aCapiMessage.Header.WordCommand of
    { ---------------------------------------------------------------}
    { The MessageNumber from CONNECT_REQ is used to find the
      corresponding CONNECT_CONF                                     }
    CONNECT_CONF : begin
      ConnectionId := aCapiMessage.Header.MessageNumber;
      Connection := FConnections.ConnectionById(ConnectionId);
      if not ConfError then begin
        Connection.State := csDConnectPending;
        Connection.PLCI := TConnectConf(aCapiMessage).PLCI;
      end else begin
        Connection.State := csDisconnected;
      end;
      if Assigned(FOnCapiEvent)
        then FOnCapiEvent(Connection, aCapiMessage, defHandling);
      { in this case no further DISCONNECT_IND is sent }
      if ConfError then FConnections.FreeConnection(Connection);
    end;
    { ---------------------------------------------------------------}
    { PLCI can be calculated out of the NCCI. The lower word of the
      NCCI is PLCI (see CAPI doc., NCCI)                             }
    CONNECT_B3_CONF : begin
      Connection := FConnections.ConnectionByPLCI(
                   TConnectB3Conf(aCapiMessage).NCCI and $0000FFFF);
      if not ConfError then begin
        Connection.State := csBConnectPending;
        Connection.NCCI  := TConnectB3Conf(aCapiMessage).NCCI;
      end else begin
         { On error reject connection, connection is released at
           DISCONNECT_IND }
        PutMessage(
          TDisconnectReq.Create(
              Connection.PLCI,
              TStruct.CreateEmpty));
      end;
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    { By default the memory area of data in B3DataQueue is released
      automatically. In this case data has to be added calling
      AddB3Data und transferred via TransferB3Data                   }
    DATA_B3_CONF : begin
      Connection :=
        FConnections.ConnectionByNCCI(TDataB3Conf(aCapiMessage).NCCI);
      if Assigned(FOnCapiEvent)
        then FOnCapiEvent(Connection, aCapiMessage, defHandling);
      { Release memory area }
      if defHandling then
        Connection.B3DataQueue.Delete(TDataB3Conf(aCapiMessage).DataHandle);
      { if connection is in transfer mode more blocks can be sent }
      if Connection.InB3DataTransferAsync then TransferB3DataAsync(Connection);
    end;
    { ---------------------------------------------------------------}
    { there's always a DISCONNECT_B3_IND after DISCONNECT_B3_CONF    }
    DISCONNECT_B3_CONF : begin
      Connection :=
        FConnections.ConnectionByNCCI(TDisconnectB3Conf(aCapiMessage).NCCI);
      Connection.State := csBDisconnectPending;
      if Assigned(FOnCapiEvent)
        then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    { there's always a DISCONNECT_IND after DISCONNECT_CONF          }
    DISCONNECT_CONF : begin
      Connection :=
        FConnections.ConnectionByPLCI(TDisconnectConf(aCapiMessage).PLCI);
      if Assigned(FOnCapiEvent)
        then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    ALERT_CONF : begin
      Connection :=
        FConnections.ConnectionByPLCI(TAlertConf(aCapiMessage).PLCI);
      if Assigned(FOnCapiEvent)
        then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    LISTEN_CONF : begin
      { no ConnectionID, as we aren't connected yet }
      if Assigned(FOnCapiEvent)
        then FOnCapiEvent(nil, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    { Connection can be nil if info isn't related to a PLCI          }
    INFO_CONF : begin
      Connection := FConnections.ConnectionByPLCI(TInfoConf(aCapiMessage).PLCI);
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    { TFacilityConf can be related to a Connection, a PLCI or an
      NCCI -> the app. has to handle this itself if it wants to }
    FACILITY_CONF : begin
      if Assigned(FOnCapiEvent)
         then FOnCapiEvent(nil, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    RESET_B3_CONF : begin
      Connection :=
        FConnections.ConnectionByNCCI(TResetB3Conf(aCapiMessage).NCCI);
      if Assigned(FOnCapiEvent)
          then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
    { ---------------------------------------------------------------}
    SELECT_B_PROTOCOL_CONF : begin
      Connection :=
        FConnections.ConnectionByPLCI(TSelectBProtocolConf(aCapiMessage).PLCI);
      if Assigned(FOnCapiEvent)
          then FOnCapiEvent(Connection, aCapiMessage, defHandling);
    end;
  end;
  CapiFreeMessage(aCapiMessage);
end;
{ -------------------------------------------------------------------}
{  TCapi.GetCapiError                                                }
{ -------------------------------------------------------------------}
Function TCapi.GetCapiError : Ansistring;
begin
  if FError > CapiInfoErrorBase
    then GetCapiError := IntToHex(FError - CapiInfoErrorBase, 4)
    else GetCapiError := '';
end;
{ -------------------------------------------------------------------}
{  TCapi.GetErrorString                                              }
{ -------------------------------------------------------------------}
Function TCapi.GetErrorString : Ansistring;
var es : Ansistring;
begin
  if FError = 0
    then es := ''
    else if FError < CapiDLLErrorBase
           then es := CapiGetInfoString(FError - CapiInfoErrorBase)
           else es := CapiGetInitStatusString(FError - CapiDLLErrorBase);
  GetErrorString := es;
end;
{ -------------------------------------------------------------------}
{  TCapi.WndProc                                                     }
{ -------------------------------------------------------------------}
procedure TCapi.WndProc(var MsgRec: TMessage);
begin
     with MsgRec do begin
         if Msg = WM_CONNECTIONS_CLEARED then begin
             if assigned(FOnConnectionsCleared)
               then FOnConnectionsCleared(self);
         end else begin
           Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
         end;
    end;
end;

{ -------------------------------------------------------------------}
{ TCapiWindowClass
  This class and the following functions allow sending messages
  via PostMessage to TCapi                                           }
{ -------------------------------------------------------------------}
var
  TCapiWindowClass : TWndClass = (
        style         : 0;
        lpfnWndProc   : @TCapiWindowProc;
        cbClsExtra    : 0;
        cbWndExtra    : SizeOf(Pointer);
        hInstance     : 0;
        hIcon         : 0;
        hCursor       : 0;
        hbrBackground : 0;
        lpszMenuName  : nil;
        lpszClassName : 'TCapiWindowClass');
{ -------------------------------------------------------------------}
{ AllocateWindowHandle
  Get a WindowHandle Windows messages are posted to.                 }
{ -------------------------------------------------------------------}
function AllocateWindowHandle(aObject : TObject) : HWND;
var
    TempClass       : TWndClass;
    ClassRegistered : Boolean;
begin
    { ClassRegistered tells if class is already registered,
      HInstance is a gloabl variable in Delphi }
    TCapiWindowClass.hInstance := HInstance;
    ClassRegistered := Windows.GetClassInfo(HInstance,
                                    TCapiWindowClass.lpszClassName,
                                    TempClass);
    if not ClassRegistered then begin
       { Register TCapiWindowClass,
         lpfnWndProc (TCapiWindowProc) is called from Windows when
         there are Windows messages.
         The class is locally registered what means there can be
         more apps. using TCapi}
       Result := Windows.RegisterClass(TCapiWindowClass);
       if Result = 0 then exit;              { leider schiefgegangen }
    end;

    { Every instance of TCapi gets his own TCapiWindowClass window }
    Result := CreateWindowEx(WS_EX_TOOLWINDOW,
                           TCapiWindowClass.lpszClassName,
                           '',           { Window name   }
                           WS_POPUP,     { Window Style  }
                           0, 0,         { X, Y          }
                           0, 0,         { Width, Height }
                           0,            { hWndParent    }
                           0,            { hMenu         }
                           HInstance,    { hInstance     }
                           nil);         { CreateParam   }

    { The window structure is extended with a pointer to the object
      to enable TCapiWindowProc to call WndProc of the corresponding
      instance of TCapi }
    if (Result <> 0) and Assigned(aObject)
       then SetWindowLong(Result, 0, Integer(aObject));
end;
{ -------------------------------------------------------------------}
{ DeallocateWindowHandle
  counterpart for CreateWindowEx. UnregisterClass mustn't be called
  here as:
  - there could be more instances of TCapi that still need the
    WindowClass
  - the WindowClass is released by Windows, when the application
    terminates                                                       }
{ -------------------------------------------------------------------}
Procedure DeallocateWindowHandle(aWindowHandle : HWnd);
begin
  DestroyWindow(aWindowHandle);
end;
{ -------------------------------------------------------------------}
{ TCapiWindowProc
  This function is called by Windows when someone posts a message
  to a WindowHandle, that was created via CreateWindowEx (see
  AllocateWindowHandle) with the class TCapiWindowClass.
  !!! This is not a method of TCapi thus is used for different
      instances of TCapi                                             }
{ -------------------------------------------------------------------}
Function TCapiWindowProc( ahWnd   : HWND;
                          auMsg   : Integer;
                          awParam : WPARAM;
                          alParam : LPARAM): Integer; stdcall;
var
    Obj    : TCapi;
    MsgRec : TMessage;
begin
    { In AllocateWindowHandle the window structure was extended with
      a pointer to the object. With GetWindowLong we get the pointer
      and are able to call Obj.WndProc }
    Obj := TCapi(GetWindowLong(ahWnd, 0));

    if not Assigned(Obj) then    { call DefWindowProc, if Obj = nil }
        Result := DefWindowProc(ahWnd, auMsg, awParam, alParam)
    else begin
        MsgRec.Msg    := auMsg;          { fill MsgRec for WndProc }
        MsgRec.wParam := awParam;
        MsgRec.lParam := alParam;
        Obj.WndProc(MsgRec);    { call WndProc of approp. instance }
        Result := MsgRec.Result;
    end;
end;

end.
