{ -----------------------------------------------------------------  }
{ CpInfo
  Author:              Peter Zwosta
  Homepage: http://home.t-online.de/home/Peter.Zwosta                }
{ -----------------------------------------------------------------  }
{ information provided by TCapiInfo can be retrieved as follows:
  - after creating the component:
    NrOfControllers, Manufacturer, Version, SerialNumber
  - after setting the property controller:
    GlobalOptions, B1ProtocolSupport, B2ProtocolSupport,
    B2ProtocolSupport
  - after the event OnSuppServInfoAvailable was fired by TCapiInfo:
    SupplementaryServices (gem. Capi Part III)
    If no event handler is provided, the component doesn't try to
    ask CAPI about supported Supplementary Services.                 }
{ -----------------------------------------------------------------  }
{ History:
  Feb 18, 1999 pz - info concerning Supported Supplementary Services }
{ -----------------------------------------------------------------  }
unit CpInfo;

interface

uses
  Windows, Messages, SysUtils, Classes, CpHand, CpConnec,
  CpMsg, CpStruct, CpConst, CpDll;

type
{ --------------------------------------------------------- }
{ TCapiProfile used in Capi_GetProfile see. CAPI20AD.DOC
  i.e. var Cp : TCapiProfile; wRes : dWord;
       wRes := Capi_GetProfile(@Cp, CtrDefault);
       label1.Caption :=
                 IntToStr(Cp.NumberOfInstalledControllers);
{ --------------------------------------------------------- }
 TCapiProfile                       = Record
    NumberOfInstalledControllers    : Word;
    NumberOfSupportedBChannels      : Word;
    GlobalOptions                   : dWord;
    B1ProtocolSupport               : dWord;
    B2ProtocolSupport               : dWord;
    B3ProtocolSupport               : dWord;
    Reserved                        : array[1..6] of dWord;
    ManufacturerSpecificInfo        : array[1..5] of dWord;
  end;
{ --------------------------------------------------------- }
{ TCapiInfo uses CAPI information functions.
  To get controller related information first set the
  property Controller. }
  TCapiInfo = class(TComponent)
  private
    FError : dword;
    FErrorString : Ansistring;
    FCapi : TCapi;
    FController : dword;
    FCapiProfile : TCapiProfile;
    FNrOfSupportedBChannels : word;
    FGlobalOptions : dword;
    FB1ProtocolSupport : dword;
    FB2ProtocolSupport : dword;
    FB3ProtocolSupport : dword;
    FSupplementaryServices : dword;
    FOnSuppServInfoAvailable : TNotifyEvent;
    procedure FSetController(aController : dword);
    procedure SetFError;
    procedure CapiEvent(aConnection : TConnection;
                        aMsg : TCapiMessage;
                        var aDefaultHandling : boolean);
  public
    constructor Create(aOwner : TComponent); override;
    destructor  Destroy; override;

    property Controller : dWord read FController write FSetController;
    property Error : dWord read FError;
    property ErrorString : Ansistring read FErrorString;

    { Available immediately after creating the component }
    function NrOfControllers : word;
    function Manufacturer : Ansistring;
    function Version : Ansistring;
    function SerialNumber : Ansistring;

    { controller dependant information,
      available after setting the property controller }
    property NrOfSupportedBChannels : word read FNrOfSupportedBChannels;
    property GlobalOptions     : dWord read FGlobalOptions;
    property B1ProtocolSupport : dWord read FB1ProtocolSupport;
    property B2ProtocolSupport : dWord read FB2ProtocolSupport;
    property B3ProtocolSupport : dWord read FB3ProtocolSupport;

    { available after the event OnSuppServInfoAvailable is fired }
    property SupplementaryServices : dWord read FSupplementaryServices;

    { see consts at the top of this units }
    function IsGlobalOptionSupported(aGlobalOption : dWord) : boolean;
    function IsB1ProtocolSupported(aB1Protocol : dWord) : boolean;
    function IsB2ProtocolSupported(aB2Protocol : dWord) : boolean;
    function IsB3ProtocolSupported(aB3Protocol : dWord) : boolean;

    { strings }
    procedure GlobalOptionsAsStrings(var OptionsAsStrings : TStringList);
    procedure B1ProtocolSupportAsStrings(var B1ProtocolSupportAsStrings : TStringList);
    procedure B2ProtocolSupportAsStrings(var B2ProtocolSupportAsStrings : TStringList);
    procedure B3ProtocolSupportAsStrings(var B3ProtocolSupportAsStrings : TStringList);
    procedure SupplementaryServicesAsStrings(var SupplementaryServicesAsStrings : TStringList);

  published
    property OnSuppServInfoAvailable : TNotifyEvent
                               read  FOnSuppServInfoAvailable
                               write FOnSuppServInfoAvailable;
  end;

const
  { consts used in  TCapiInfo.Is...Supported ------------------------}
  { Global Options }
  goInternalEquipment     : dword = $0001;
  goExternalEquipment     : dword = $0002;
  goHandset               : dword = $0004;
  goDTMF                  : dword = $0008;
  goSupplementaryServices : dword = $0010;
  goChannelAllocation     : dword = $0020;
  { B1Protocol --------------------------}
  b164kHDLC               : dword = $0001;
  b164kTransparent        : dword = $0002;
  b1V110Asynchronous      : dword = $0004;
  b1V110Synchronous       : dword = $0008;
  b1T30FaxG3              : dword = $0010;
  b164kInvertedHDLC       : dword = $0020;
  b156kTransparent        : dword = $0040;
  b1Modem                 : dword = $0080;
  b1ModemAsynchronous     : dword = $0100;
  b1ModemSynchronousHDLC  : dword = $0200;
  { B2Protocol --------------------------}
  b2ISO7776               : dword = $0001;
  b2Transparent           : dword = $0002;
  b2SDLC                  : dword = $0004;
  b2LAPD                  : dword = $0008;
  b2T30FaxG3              : dword = $0010;
  b2PPP                   : dword = $0020;
  b2TransparentIgnoreFErr : dword = $0040;
  b2ModemV42bisMNP5       : dword = $0080;
  b2ISO7776V42bis         : dword = $0100;
  b2V120Asynchronous      : dword = $0200;
  b2V120AsnychronousV42bis : dword = $0400;
  b2V120Transparent       : dword = $0800;
  b2LAPDWithSAPISel       : dword = $1000;
  { B3Protocol --------------------------}
  b3Transparent           : dword = $0001;
  b3T90NL                 : dword = $0002;
  b3ISO8202               : dword = $0004;
  b3X25                   : dword = $0008;
  b3T30FaxG3              : dword = $0010;
  b3T30FaxG3Ext           : dword = $0020;
  b3Modem                 : dword = $0080;
  { Supplementary Services - return values of Get Supported Services }
  cssHoldRetrieveSupported        : dword = $0001;           { bit 0 }
  cssTerminalPortabilitySupported : dword = $0002;           { bit 1 }
  cssECTSupported                 : dword = $0004;           { bit 2 }
  css3PTYSupported                : dword = $0008;           { bit 3 }
  cssCallForwardingSupported      : dword = $0010;           { bit 4 }
  cssCallDeflectionSupported      : dword = $0020;           { bit 5 }

procedure Register;

implementation
{ -------------------------------------------------------------------}
{ TCapiInfo.Create                                                   }
{ -------------------------------------------------------------------}
constructor TCapiInfo.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  FCapi := nil;
  FError := 0;
  FNrOfSupportedBChannels := 0;
  FGlobalOptions          := 0;
  FB1ProtocolSupport      := 0;
  FB2ProtocolSupport      := 0;
  FB3ProtocolSupport      := 0;
  FSupplementaryServices  := 0;
  FOnSuppServInfoAvailable := nil;
  FController := 0;
  FillChar(FCapiProfile, SizeOf(FCapiProfile), #0);
  { load Capi2032.dll }
  CapiInitDLL;
  if CapiGetInitStatus <> ciOK then SetFError;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.Destroy                                                  }
{ -------------------------------------------------------------------}
destructor TCapiInfo.Destroy;
begin
  if assigned(FCapi) then FCapi.Free;
  inherited destroy;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.FSetController                                           }
{ -------------------------------------------------------------------}
procedure TCapiInfo.FSetController(aController : dword);
begin
  FController := aController;

  { Capi_GetProfile is a function provided by CAPI2032.DLL }
  FError := Capi_GetProfile(@FCapiProfile, FController);
  if (FError <> 0) then exit;
  FNrOfSupportedBChannels := FCapiProfile.NumberOfSupportedBChannels;
  FGlobalOptions := FCapiProfile.GlobalOptions ;
  FB1ProtocolSupport := FCapiProfile.B1ProtocolSupport;
  FB2ProtocolSupport := FCapiProfile.B2ProtocolSupport;
  FB3ProtocolSupport := FCapiProfile.B3ProtocolSupport;

  { to get information about Supplementary Services we must send
    a FACILITY_REQ to CAPI }
  if not IsGlobalOptionSupported(goSupplementaryServices) then exit;
  { if the app. doesn't provide the event we don't send FACILITY_REQ }
  if not assigned(FOnSuppServInfoAvailable) then exit;

  if assigned(FCapi) then FCapi.Free;
  FCapi := TCapi.Create(1, 2, 128);
  SetFerror;
  if FError <> 0 then begin
    FCapi.Free;
    FCapi := nil;                       { for correct use assigned() }
    exit;
  end;
  FCapi.OnCapiEvent := CapiEvent;    { event handler for CAPI-Events }

  if not FCapi.PutMessage(
    TFacilityReq.Create(
       FController,
       fsSupplementaryService,
       TSuppServFacilityReqParameter.Create_GetSupportedServices)) then begin
    SetFError;
    FCapi.Free;
    FCapi := nil;                       { for correct use assigned() }
  end;
end;
{ -------------------------------------------------------------------}
{  TCapiInfo.SetFError                                               }
{ -------------------------------------------------------------------}
procedure TCapiInfo.SetFError;
begin
  if Assigned(FCapi) then begin
    FError := FCapi.Error;
    if FCapi.ErrorString <> ''
      then FErrorString := FCapi.ErrorString
      else FErrorString := 'OK';
  end else begin
    FError := CapiGetInitStatus;
    FErrorString := CapiGetInitStatusString(FError);
  end;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.CapiEvent                                                }
{ -------------------------------------------------------------------}
procedure TCapiInfo.CapiEvent(aConnection : TConnection;
                              aMsg : TCapiMessage;
                              var aDefaultHandling : boolean);
var tss : TSuppServFacilityConfParameter;
    ServiceInfo : word;
begin
  SetFError;

  case aMsg.Header.WordCommand of
    { ---------------------------------------------------------------}
    FACILITY_CONF : begin
      with TFacilityConf(aMsg) do begin
        if (FacilitySelector = fsSupplementaryService) then begin
          tss := TSuppServFacilityConfParameter.
                 CreateFromData(FacilityParameters.Header);
          tss.GetSupportedServices(ServiceInfo, FSupplementaryServices);
          tss.Free;
          if assigned(FOnSuppServInfoAvailable) then FOnSuppServInfoAvailable(self);
        end;
      end; { with }
    end; { FACILITY_CONF }
  end; { case }
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.NrOfControllers
  Find out the number of installed controllers. If Capi_GetProfile
  gets parameter 0 as value for controller only the number of
  controller is set by CAPI.                                         }
{ -------------------------------------------------------------------}
Function TCapiInfo.NrOfControllers : word;
var Cp   : TCapiProfile;
begin
  FError := Capi_GetProfile(@Cp, 0);
  NrOfControllers := Cp.NumberOfInstalledControllers;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.Manufacturer                                             }
{ -------------------------------------------------------------------}
function TCapiInfo.Manufacturer : Ansistring;
var manu : Ansistring;
begin
  SetLength(Manu, 64);
  Capi_GetManufacturer(PChar(Manu));
  FError := 0; // Get_Manufacturer doesn't set an error
  Manufacturer := manu;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.Version
  If Version doesn't show the same value than the ISDN software of
  your manufacturer, the reason is that different manufacturers use
  their own coding for mMaj and mMin.                                }
{ -------------------------------------------------------------------}
function TCapiInfo.Version : Ansistring;
var cMaj, cMin, mMaj, mMin : dword;

begin
  FError := Capi_GetVersion(cMaj, cMin, mMaj, mMin);
  { I inserted the following casts as there seemed to be CAPIs
    returning a word instead of a dword. In these cases the upper
    word is by chance. This is not a very nice solution but it works
    as the version normally isn't bigger than a word. I guess the
    problem is related to the fact that under Win 3.xx CAPI defines
    this parameter as word and not dword. }
  if (cMaj > MaxWord) then begin
    cMaj := word(cMaj);
    cMin := word(cMin);
    mMaj := word(mMaj);
    mMin := word(mMin);
  end;
  { set version string }
  Version :=
     'CAPI: ' + intToStr(cMaj) + '.' + intToStr(cMin) +
     '  Version: ' + intToStr(mMaj) + '.' + intToStr(mMin);
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.SerialNumber                                             }
{ -------------------------------------------------------------------}
function TCapiInfo.SerialNumber : Ansistring;
var SNr : Ansistring;
begin
  SetLength(Snr, 8);
  FError := Capi_GetSerialNumber(pChar(SNr));
  SerialNumber := SNr;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.IsGlobalOptionSupported                                  }
{ -------------------------------------------------------------------}
function TCapiInfo.IsGlobalOptionSupported(aGlobalOption : dWord) : boolean;
begin
  IsGlobalOptionSupported :=
              (FCapiProfile.GlobalOptions and aGlobalOption) > 0;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.IsB1ProtocolSupported                                    }
{ -------------------------------------------------------------------}
function TCapiInfo.IsB1ProtocolSupported(aB1Protocol : dWord) : boolean;
begin
  IsB1ProtocolSupported :=
              (FCapiProfile.B1ProtocolSupport and aB1Protocol) > 0;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.IsB2ProtocolSupported                                    }
{ -------------------------------------------------------------------}
function TCapiInfo.IsB2ProtocolSupported(aB2Protocol : dWord) : boolean;
begin
  IsB2ProtocolSupported :=
              (FCapiProfile.B2ProtocolSupport and aB2Protocol) > 0;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.IsB3ProtocolSupported                                    }
{ -------------------------------------------------------------------}
function TCapiInfo.IsB3ProtocolSupported(aB3Protocol : dWord) : boolean;
begin
  IsB3ProtocolSupported :=
              (FCapiProfile.B3ProtocolSupport and aB3Protocol) > 0;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.GlobalOptionsAsStrings
  Resolves the options dword to strings                              }
{ -------------------------------------------------------------------}
procedure TCapiInfo.GlobalOptionsAsStrings(var OptionsAsStrings : TStringList);
var go : dWord;
begin
  go := FCapiProfile.GlobalOptions;
  with OptionsAsStrings do begin
    if go and $0001 > 0 then Add('[0] internal controller supported');
    if go and $0002 > 0 then Add('[1] external equipment supported');
    if go and $0004 > 0 then Add('[2] Handset supported');
    if go and $0008 > 0 then Add('[3] DTMF supported');
    if go and $0010 > 0 then Add('[4] Supplementary Services');
    if go and $0020 > 0 then Add('[5] channel allocation supported');
  end;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.B1ProtocolSupportAsStrings                               }
{ -------------------------------------------------------------------}
procedure TCapiInfo.B1ProtocolSupportAsStrings(var B1ProtocolSupportAsStrings : TStringList);
var bs : dWord;
begin
  bs := FCapiProfile.B1ProtocolSupport;
  with B1ProtocolSupportAsStrings do begin
    if bs and $0001 > 0 then Add('[0] 64 kbit/s with HDLC framing');
    if bs and $0002 > 0 then Add('[1] 64 kbit/s bit-transparent operation with byte framing from the network');
    if bs and $0004 > 0 then Add('[2] V.110 asynchronous operation with start/stop byte framing');
    if bs and $0008 > 0 then Add('[3] V.110 synchronous operation with HDLC framing');
    if bs and $0010 > 0 then Add('[4] T.30 modem for fax group 3');
    if bs and $0020 > 0 then Add('[5] 64 kbit/s inverted with HDLC framing');
    if bs and $0040 > 0 then Add('[6] 56 kbit/s bit-transparent operation with byte framing from the network');
    if bs and $0080 > 0 then Add('[7] Modem with all negotiations');
    if bs and $0100 > 0 then Add('[8] Modem asynchronous operation with start/stop byte framing');
    if bs and $0200 > 0 then Add('[9] Modem synchronous operation with HDLC framing');
  end;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.B12ProtocolSupportAsStrings                              }
{ -------------------------------------------------------------------}
procedure TCapiInfo.B2ProtocolSupportAsStrings(var B2ProtocolSupportAsStrings : TStringList);
var bs : dWord;
begin
  bs := FCapiProfile.B2ProtocolSupport;
  with B2ProtocolSupportAsStrings do begin
    if bs and $0001 > 0 then Add('[0] ISO 7776 (X.75 SLP)');
    if bs and $0002 > 0 then Add('[1] Transparent');
    if bs and $0004 > 0 then Add('[2] SDLC');
    if bs and $0008 > 0 then Add('[3] LAPD in accordance with Q.921 for D-channel X.25 (SAPI 16)');
    if bs and $0010 > 0 then Add('[4] T.30 for fax group 3');
    if bs and $0020 > 0 then Add('[5] Point-to-Point Protocol (PPP)');
    if bs and $0040 > 0 then Add('[6] Transparent (ignoring framing errors of B1 protocol)');
    if bs and $0080 > 0 then Add('[7] Modem error correction and compression (V.42 bis or MNP5)');
    if bs and $0100 > 0 then Add('[8] ISO 7776 (X.75 SLP) modified supporting V.42 bis compression');
    if bs and $0200 > 0 then Add('[9] V.120 asynchronous mode');
    if bs and $0400 > 0 then Add('[10] V.120 asynchronous mode supporting V.42 bis');
    if bs and $0800 > 0 then Add('[11] V.120 bit-transparent mode');
    if bs and $1000 > 0 then Add('[12] LAPD in accordance with Q.921 including free SAPI selection');
  end;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.B3ProtocolSupportAsStrings                               }
{ -------------------------------------------------------------------}
procedure TCapiInfo.B3ProtocolSupportAsStrings(var B3ProtocolSupportAsStrings : TStringList);
var bs : dWord;
begin
  bs := FCapiProfile.B3ProtocolSupport;
  with B3ProtocolSupportAsStrings do begin
    if bs and $0001 > 0 then Add('[0] Transparent');
    if bs and $0002 > 0 then Add('[1] T.90NL with compatibility to T.70NL in accordance to T.90 Appendix II');
    if bs and $0004 > 0 then Add('[2] ISO 8208 (X.25 DTE-DTE)');
    if bs and $0008 > 0 then Add('[3] X.25 DCE');
    if bs and $0010 > 0 then Add('[4] T.30 for fax group 3');
    if bs and $0020 > 0 then Add('[5] T.30 for fax group 3 with extensions');
    { if bs and $0040 then Add('reserved'); }
    if bs and $0080 > 0 then Add('[7] Modem');
  end;
end;
{ -------------------------------------------------------------------}
{ TCapiInfo.SupplementaryServicesAsStrings
  Delivers the strings for Facility Confirmation Parameter,
  Supplementary Service, function Get Supported Services the return
  value Supported Services as strings (see. Capi, Part III,
  Supplementary Services)                                            }
{ -------------------------------------------------------------------}
procedure TCapiInfo.SupplementaryServicesAsStrings(var SupplementaryServicesAsStrings : TStringList);
var ss : dWord;
begin
  ss := FSupplementaryServices;
  with SupplementaryServicesAsStrings do begin
    if (ss and cssHoldRetrieveSupported) > 0        then Add('[0] Hold / Retrieve');
    if (ss and cssTerminalPortabilitySupported) > 0 then Add('[1] Terminal Portability');
    if (ss and cssECTSupported) > 0                 then Add('[2] ECT');
    if (ss and css3PTYSupported) > 0                then Add('[3] 3PTY');
    if (ss and cssCallForwardingSupported) > 0      then Add('[4] Call-Forwarding');
    if (ss and cssCallDeflectionSupported) > 0      then Add('[5] Call-Deflection');
  end;
end;

{ -------------------------------------------------------------------}
{  Register
   component in the delphi component palette                         }
{ -------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('Capi', [TCapiInfo]);
end;

end.
