{ -----------------------------------------------------------------  }
{ CpMoni
  Author:              Peter Zwosta
  Homepage: http://home.t-online.de/home/Peter.Zwosta                }
{ -----------------------------------------------------------------  }
{ METHODS
  --------------------------------------------------------------------
  WaitForCall: the call monitor starts waiting for incoming calls
    - Controller is the number of the ISDN-device (default 1)
    - RejectTimeOut determines the time, a user (or the application
      itself) is able to react upon an incoming call.
      If the application sets RejectTimeOut > 0 it can easily realize
      rejection of incoming calls on a per call basis (i.e. as a
      reaction to the calling party number).
      RejectCall or ClearCall are used to refuse the call.
  RejectCall
      By means of this function an incoming call is rejected without
      telephone charges for the caller.
      WARNING !!!
      This function doesn't work as intended if there's another isdn
      devices ready to take the call. I.e. if there's another
      isdn telephone waiting for a call, rjeUserBusy just means that
      the call monitor can't accept the call. If the telephone doesn't
      also say it was occupied, the caller won't receive occupied.
      Same thing is with an answercall who accepts the call.
      Thus in most real world environments RejectCall isn't very
      useful.
  ClearCall
      With ClearCall you can be sure the caller won't come through.
      WARNING !!!
      In ClearCall the callmonitor accepts the call and then
      immediately hangs up. The caller has to pay charges!!!
      If the caller uses an isdn telephone he might realize that the
      call was accepted for a short time (though he certainly doesn't
      know what happened). If he doesn't, he normally guesses the line
      is occupied.
  Reset
      The application is unregistered with CAPI, CAPI2032.DLL is
      unloaded, loaded again and the application is registered
      again. The application must call WaitForCall again after having
      called Reset if it wants to be informed about future incoming
      calls.
      For some isdn devices it's necessary to call reset after
      having received an incoming call.
      Reset should only be called when the event OnResetCapi is fired.
  --------------------------------------------------------------------
  PROPERTIES
  --------------------------------------------------------------------
  NoDisturbMode:
      If this property is set to true, the component calls ClearCall
      upon an incoming call itself. Any incoming calls are rejected
      and the caller has to pay for it.
      If NoDisturbMode is set to true RejectTimeOut is ignored.
  --------------------------------------------------------------------
  EVENTS
  --------------------------------------------------------------------
  There's no more than a few seconds (3-4) time for an application to
  deal with the events. If the application needs more than a few
  seconds to find out i.e. the name related to a phone number it can
  use PostMessage in OnIncomingCall to enable the component to return
  to the thread waiting for capi messages.

  OnRejectCallQuestion:
      This event enables the application to reject incoming calls
      immediately without asking the user (i.e. the user can provide
      a list of phonenumbers of "persona non gratas" he wants the app.
      to reject). The caller has to pay for it (That's what the user
      wants those people to do :-)
      the app. can change the parameter RejectCause using the Consts
      in CpConst (i.e. rjeAcceptCall to Accept the call and hang up
      immediately).
      Notice !! What the other rje... are concerned see the comments
      for the method RejectCall
  OnIncomingCall:
      The application provides an event handler for this event to
      show
      - calling party number
      - called party number and
      - cip-value of an incoming call.
      - the RejectCause if a CONNECT_RESP was sent to CAPI already
        (see CpConst rje... for valid causes)
        Note!!! rjeAcceptCall means that the call was accepted by
                the component but will be discarded immediately.
                The caller usually hears a busy signal and has to
                pay for it.
  OnDisconnectInfo:
      This event is sent when Capi sends a DISCONNECT_IND.
      If aRejectCause = rjeUndefined then we did'nt answer CONNECT_IND
      yet but another event occured that lead to a DISONNECT_IND,
      i.e. reason $349A - non selected user clearing means that another
      isdn device or application accepted the call
      You can use CapiGetInfoString (see CpConst.pas) to get the
      text related to aDisconnectReason.
      To receive valuable information in aDisconnectReason you have
      to call TCapiMonitor.WaitForCall with a rejectTimout > 0.
      If you don't CONNECT_IND will be answered immediately and CAPI
      immediately sends a DISCONNECT_IND.
  OnStateChange:
      An event handler for this function is called when the user or
      the application are allowed to call RejectCall (bUserCanReject =
      true) or to change the property NoDisturbMode
      (bUserCanCangeNoDisturbMode = true). The event handler is called
      when the status changes from allowed to not allowed vice versa.
  OnDoResetCapi
      This event handler is called, when the application is allowed
      to call Reset. The event is fired when an incoming call is
      handled (see CpHand for further information).
{ -----------------------------------------------------------------  }
{ History:
  Oct 8, 1999 pz - Event OnRejectCallQuestion added. Added
      OnDisconnectInfo to inform the app. about reject cause or
      disconnect reason
      - Problem with missing leading zero in CallingPartyNumber fixed 
      (see TCapiMonitor.CapiEvent  CONNECT_IND)
  Nov 1, 1998 pz - Event OnDoResetCapi added, to enable an application
      to reset CAPI after an incoming call. This is necessary for
      some isdn devices.                                             }
{ -----------------------------------------------------------------  }
unit CpMoni;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls,
  Forms, Dialogs, CpHand, CpMsg, CpStruct, CpConst, CpConnec;

type
  { Event: an incoming call arrives }
  TCapiCallNotification = procedure (CallingPartyNumber : Ansistring;
                                     CalledPartyNumber : Ansistring;
                                     CipValue : word;
                                     CipValueString : Ansistring)
                             of object;
  TRejectCallQuestion   = procedure (CallingPartyNumber : Ansistring;
                                     CalledPartyNumber : Ansistring;
                                     CipValue : word;
                                     var aRejectCause : word)
                             of object;
  TDisconnectInfo = procedure (aRejectCause : word;
                               aDisconnectReason : word)
                             of object;
  { Info about whether values for CONNECT_RESP can be changed or not
    or an error has occured }
  TStateChange = procedure(bUserCanReject : boolean;
                           bUserCanCangeNoDisturbMode : boolean) of object;

  { -----------------------------------------------------------------}
  TCapiMonitor =  class(TComponent)
  private
    FCapi           : TCapi;
    FController     : dword;
    FRejectTimeOut  : integer;         { determines the time a until
                                  CONNECT_RESP is sent automatically }
    FNoDisturbMode  : boolean;
    FSecondsToReject : integer;     { Nr. of sec., until CONNECT_RESP
                                                is sent automatically}
    FLastRejectCause : word;
    FTimer           : TTimer;                  { counts the seconds }
    FError           : dword;
    FErrorString     : Ansistring;
    FOnRejectCallQuestion : TRejectCallQuestion;
    FOnIncomingCall  : TCapiCallNotification;
    FOnStateChange   : TStateChange;
    FOnDisconnectInfo : TDisconnectInfo;
    FOnDoResetCapi   : TNotifyEvent;

    procedure CapiEvent(aConnection : TConnection;
                        aMsg : TCapiMessage;
                        var aDefaultHandling : boolean);
    procedure ConnectionsCleared(Sender : TObject);
    procedure OnTimerTick(Sender : TObject);
    procedure SetFError;
  public
    constructor Create(aOwner : TComponent); override;
    function    WaitForCall(aController : dWord;
                            aRejectTimeout : integer) : boolean;
    procedure   RejectCall(aCause : word);
    procedure   ClearCall;
    function    ResetCapi : boolean;
    destructor  Destroy; override;
  published
    property RejectTimeOut  : integer read FRejectTimeOut;
    property Controller     : dWord read FController;
    property Error : dword read FError;
    property ErrorString : Ansistring read FErrorString;
    property NoDisturbMode : boolean read FNoDisturbMode write FNoDisturbMode default false;
    property LastRejectCause : word read FLastRejectCause; 

    property OnRejectCallQuestion : TRejectCallQuestion read FOnRejectCallQuestion write FOnRejectCallQuestion;
    property OnIncomingCall : TCapiCallNotification read FOnIncomingCall write FOnIncomingCall;
    property OnStateChange : TStateChange read FOnStateChange write FOnStateChange;
    property OnDisconnectInfo : TDisconnectInfo read FOnDisconnectInfo write FOnDisconnectInfo;
    property OnDoResetCapi  : TNotifyEvent read FOnDoResetCapi write FOnDoResetCapi;
  end;

procedure Register;

implementation

{ -------------------------------------------------------------------}
{  TCapiMonitor.Create                                               }
{ -------------------------------------------------------------------}
constructor TCapiMonitor.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  FError := 0;
  FErrorString := '';
  FController := ctrDefault;
  FRejectTimeOut := 0;
  FSecondsToReject := 0;
  FNoDisturbMode := false;
  FLastRejectCause := rjeUndefined; { see cpConst.pas } 

  FTimer := nil;
  FOnRejectCallQuestion := nil;
  FOnIncomingCall := nil;
  FOnStateChange := nil;
  FOnDisconnectInfo := nil;
  FOnDoResetCapi := nil;
end;
{ -------------------------------------------------------------------}
{  TCapiMonitor.Destroy                                        }
{ -------------------------------------------------------------------}
destructor TCapiMonitor.Destroy;
begin
  if Assigned(FCapi) then FCapi.Free;
  if Assigned(FTimer) then FTimer.Free;
  inherited destroy;
end;
{ -------------------------------------------------------------------}
{  TCapiMonitor.WaitForCall                                     }
{ -------------------------------------------------------------------}
function TCapiMonitor.WaitForCall(aController : dWord;
                                       aRejectTimeout : integer) : boolean;
begin
  FError := 0;
  If Assigned(FCapi) then FCapi.Free;
  FController := aController;
  FRejectTimeout := aRejectTimeout;

  { Create TCapi means to load CAPI2032.DLL and register application
    We accept just one logical connection at a time }
  FCapi := TCapi.Create(1, 2, 128);

  SetFError;
  if FError <> 0 then begin
    FCapi.Free;
    FCapi := nil;       { enable for use with assigned }
    WaitForCall := false;
    exit;
  end;
  FCapi.OnCapiEvent := CapiEvent;    { event handler for CAPI-events }
  FCapi.OnConnectionsCleared := ConnectionsCleared;

  { send LISTEN_REQ -> application is informed about incoming calls
    see CpConst.pas for the consts }
  if not FCapi.PutMessage(
    TListenReq.Create( FController, imaMaskAll, cipDefault, cip2Default,
      TCallingPartyNumber.Create(tonDefault, sidDefault, ''),
      TCallingPartySubAddress.Create(tosDefault, '')))
  then begin
    SetFError;
    FCapi.Free;
    FCapi := nil;      { enable for use with assigned }
    if Assigned(FOnStateChange) then FOnStateChange(false, false);
  end else begin
    if Assigned(FOnStateChange) then FOnStateChange(false, true);
  end;
  WaitForCall := (FError = 0);
end;
{ -------------------------------------------------------------------}
{  TCapiMonitor.CapiEvent
   CapiEvent takes the events from TCapi and answers them.
   FOnIncomingCall is fired when a CONNECT_IND is received.
   aDefaultHandling is changed only when CONNECT_B3_IND is answered
   as default of TCapi is to accept the call.                        }
{ -------------------------------------------------------------------}
procedure TCapiMonitor.CapiEvent(aConnection : TConnection;
                                      aMsg : TCapiMessage;
                                      var aDefaultHandling : boolean);
var RejectCause     : word;
    xCallingPartyNumber : Ansistring; 
begin
  SetFError;                         { set error for the application }

  case aMsg.Header.WordCommand of
    { ---------------------------------------------------------------}
    LISTEN_CONF : begin
      { if there are erros, the application should know about them }
      If Assigned(FOnStateChange) then FOnStateChange(false, true);
    end;
    { ---------------------------------------------------------------}
    CONNECT_IND : begin
        aConnection.Tag := dword(self);             { see RejectCall }

        xCallingPartyNumber := TConnectInd(aMsg).CallingPartyNumber.Contents;
        { if TypeOfNumber is national or international number
          (ETS 300 102 4.5.10) there's no leading zero so we add it  }
        if (xCallingPartyNumber <> '') then begin
          if (Pos('0', xCallingPartyNumber) <> 1) and
             ((TConnectInd(aMsg).CallingPartyNumber.TypeOfNumber and
               tonInternationalNumberMask) > 0) or
             ((TConnectInd(aMsg).CallingPartyNumber.TypeOfNumber and
               tonNationalNumberMask) > 0) then
          xCallingPartyNumber := '0' + xCallingPartyNumber; 
        end;
        { inform application about incoming call and number etc.
          DefaultHandling = false and RejectCause enables the
          application to react immediately on special combinations
          of calling party, called party and cip value }
        RejectCause := rjeUndefined;  { see CpConst for valid causes }
        FLastRejectCause := rjeUndefined;
        if Assigned(FOnRejectCallQuestion)
          then FOnRejectCallQuestion(xCallingPartyNumber,
                             TConnectInd(aMsg).CalledPartyNumber.Contents,
                             TConnectInd(aMsg).CipValue,
                             RejectCause);

        if (RejectCause <> rjeUndefined) then begin
          RejectCall(RejectCause); { Immediate reject by application }
        end else begin
           if FNoDisturbMode then begin            { accept the call }
             RejectCall(rjeAcceptCall);
           end else begin
             { start Timer if FRejectCall > 0 }
             if FRejectTimeOut > 0 then begin
               FSecondsToReject := FRejectTimeout;
               if not Assigned(FTimer) then FTimer := TTimer.Create(self);
               FTimer.OnTimer := OnTimerTick;
               FTimer.Interval := 1000; { each second }
               { ALERT_REQ gives the user 120 sec. of time to decide
                 what do to }
               FCapi.PutMessage(
                   TAlertReq.Create(
                      aConnection.PLCI,
                      TStruct.CreateEmpty));
             end else begin { default is to ignore the call }
               RejectCall(rjeIgnoreCall);
             end; // if FRejectTimeOut
           end; // if FNoDisturbMode
        end; // if DefaultHandling

        { inform user about status change - User can't change
          FNoDisturbMode at this time, but the user can Reject the
          call by menu if the application has set RejectTimeOut > 0 }
        if Assigned(FOnStateChange)
          then FOnStateChange((RejectCause = rjeUndefined) and
                              (FRejectTimeOut > 0) and not FNoDisturbMode,
                             false);

        { inform user about incoming call }
        if Assigned(FOnIncomingCall)
          then FOnIncomingCall(xCallingPartyNumber,
                               TConnectInd(aMsg).CalledPartyNumber.Contents,
                               TConnectInd(aMsg).CipValue,
                               CapiGetCipValueString(TConnectInd(aMsg).CipValue));
      end;
    { ---------------------------------------------------------------}
    CONNECT_B3_IND : begin
      aDefaultHandling := false;          { don't send rjeAcceptCall }

      FCapi.PutMessage(                                { reject call }
        TConnectB3Resp.Create(
          aConnection.NCCI,
          rjeB3NormalClear,
          TStruct.CreateEmpty));

      FCapi.Disconnect(aConnection);              {  ... and hang up }
      if Assigned(FOnStateChange) then FOnStateChange(false, false);
    end;
    { ---------------------------------------------------------------}
    DISCONNECT_IND : begin   { DISCONNECT_RESP is performed by TCapi }
        FSecondsToReject := 0;
        if Assigned(FTimer) then begin
           FTimer.Free;
           FTimer := nil;
        end;
        if Assigned(FOnDisconnectInfo)
          then FOnDisconnectInfo(FLastRejectCause,
                                 TDisconnectInd(aMsg).Reason);
        if Assigned(FOnStateChange) then FOnStateChange(false, true);
     end;
  end;
end;
{ -------------------------------------------------------------------}
{ TCapiMonitor.ConnectionsCleared                               }
{ -------------------------------------------------------------------}
procedure TCapiMonitor.ConnectionsCleared(Sender : TObject);
begin
  if assigned(FOnDoResetCapi) then FOnDoResetCapi(self);
end;
{ -------------------------------------------------------------------}
{  TCapiMonitor.RejectCall
   Rejects the desired connection by using CONNECT_RESP with <cause> }
{ -------------------------------------------------------------------}
procedure TCapiMonitor.RejectCall(aCause : word);
var aConnection : TConnection;
begin
  { if TCapi.Create was not successful then exit }
  if not Assigned(FCapi) then exit;
  { if meanwhile the connection was cut from the another side }
  aConnection := FCapi.Connections.ConnectionByTag(dWord(self));
  if not Assigned(aConnection) then exit;
  { there must have been a CONNECT_IND -> csDConnectPending }
  if aConnection.State <> csDConnectPending then exit;
  { user can't change values from now on }
  if Assigned(FOnStateChange) then FOnStateChange(false, false);
  { Free timer, the timer is created again at the next CONNECT_IND }
  FSecondsToReject := 0;
  if Assigned(FTimer) then begin
     FTimer.Free;
     FTimer := nil;
  end;

  FLastRejectCause := aCause;
  
  if (aCause = rjeAcceptCall) then begin
    FCapi.PutMessage(                              { accept the call }
      TConnectResp.Create(    { the other work is performed by TCapi } 
         aConnection.PLCI,
         rjeAcceptCall,
         { B1-protocol: 64 kBit/s bit-transparent ($0001)
           B2-protocol: Transparent               ($0001)
           B3-protocol: Transparent               ($0000) }
         TBProtocol.Create($0001,$0001,$0000, TStruct.CreateEmpty,
                       TStruct.CreateEmpty,TStruct.CreateEmpty),
         TConnectedNumber.CreateEmpty,
         TConnectedSubAddress.CreateEmpty,
         TStruct.CreateEmpty,
         TStruct.CreateEmpty));
  end else begin
    FCapi.PutMessage(        { send CONNECT_RESP  with desired cause }
      TConnectResp.Create(  
        aConnection.PLCI,
        aCause,
        TBProtocol.CreateEmpty,
        TConnectedNumber.CreateEmpty,
        TConnectedSubAddress.CreateEmpty,
        TStruct.CreateEmpty,
        TStruct.CreateEmpty));
  end;
end;
{ -------------------------------------------------------------------}
{  TCapiMonitor.ClearCall
   Accepts the call and after that immediately hangs up.             }
{ -------------------------------------------------------------------}
procedure TCapiMonitor.ClearCall;
begin
  RejectCall(rjeAcceptCall);
end;
{ -------------------------------------------------------------------}
{ TCapiMonitor.ResetCapi
  Unregisters the application, unloads DLL, loads DLL, registers
  application again                                                  }
{ -------------------------------------------------------------------}
function TCapiMonitor.ResetCapi : boolean;
begin
  if assigned(FCapi)
    then ResetCapi := FCapi.Reset(true)
    else ResetCapi := false;
end;
{ -------------------------------------------------------------------}
{  TCapiMonitor.SetFError                                       }
{ -------------------------------------------------------------------}
procedure TCapiMonitor.SetFError;
begin
  if Assigned(FCapi) then begin
    FError := FCapi.Error;
    if FCapi.ErrorString <> ''
      then FErrorString := FCapi.ErrorString
      else FErrorString := 'OK';
  end;
end;
{ -------------------------------------------------------------------}
{  TCapiMonitor.OnTimer                                         }
{ -------------------------------------------------------------------}
procedure TCapiMonitor.OnTimerTick(Sender : TObject);
begin
  dec(FSecondsToReject);
  if FSecondsToReject <= 0 then RejectCall(rjeIgnoreCall);
end;
{ -------------------------------------------------------------------}
{  Register
   Register component in the commponent collection of Delphi         }
{ -------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('Capi', [TCapiMonitor]);
end;

end.
