{ -----------------------------------------------------------------  }
{ CpStruct
  Author:              Peter Zwosta
  Homepage: http://home.t-online.de/home/Peter.Zwosta                }
{ -----------------------------------------------------------------  }
{ this unit declares and implements the structs (see CAPi doc.) as
  parameters of CAPi messages
{ -----------------------------------------------------------------  }
{
  TStruct                                       parent of all structs
  TCallingPartyNumber
  TCallingPartySubAddress
  TCalledPartyNumber
  TCalledPartySubAddress
  TConnectedNumber
  TConnectedSubAddress
  TBProtocol
  TCapiString                                        20.11.98 pz, rb
  TDTMFDigits                                             -"-
  TDTMFFacilityReqParameter                               -"-
  TDTMFFacilityConfParameter                              -"-
  TDTMFFacilityIndParameter                               -"-
  TDTMFFacilityRespParameter                              -"-
  TSuppServFacilityReqParameter                      16.02.99 pz
  TSuppServFacilityConfParameter                          -"-
  TSuppServFacilityIndParameter                           -"-
  TSuppServFacilityRespParameter                          -"-
{ -----------------------------------------------------------------  }
{ History:
  Feb 16, 1999 pz - TDTMFFacilityReqParameter Member variables
       deleted, TSuppServFacilityXXXParameter added                  }
{ -----------------------------------------------------------------  }
unit CpStruct;

interface
uses SysUtils, Windows, CpConst;
type
{ -----------------------------------------------------------------  }
{ TStruct
  Parent of all Structs (parameters for CAPI messages, see CAPI doc.)
  - The first byte of a struct, the length byte, determines the length
    of the following data structure. If the data area is longer than
    255 bytes, the first byte is set to 255 whereas the following word
    is set to the length of the struct data.
  - TStruct fills FStruct following this struct definition
  - The property Header is a pointer to the structure and is used to
         insert a struct into a CAPI message
  - Length is the length of the structure including the first byte
         resp. byte+word. Length is used to calculate the length of
         a CAPI message (see CpMsg.pas - message.Create methods).
  - DataLenght is the length without the lenght byte resp. byte+word
  - Create, CreateEmpty, MakeStruct and AddStructData are used with
         messages being sent to CAPI
  - CreateFromData and GetStructData is used with messages from CAPI
{ -----------------------------------------------------------------  }
 TStruct = class(TObject)
     FStruct     : pByte;
     FStructPtr  : pByte;
     bFreeStruct : boolean;
     FBytesRead  : word;
     Function  GetDataLength : word;
     Function  GetLength : word;
  public
     Procedure MakeStruct(aDataLength : word);
     Procedure AddStructData(aDataPtr : pointer; aDataLength : word);
     Procedure GetStructData(aDataPtr : pointer; aDataLength : word);
     Procedure Reset;

     constructor Create;
     constructor CreateEmpty;
     constructor CreateFromData(aDataPtr : pointer);
     destructor  Destroy; override;

     property Header     : pByte read FStruct;
     property Length     : word read GetLength;
     property DataLength : word read GetDataLength;
 end;
{ -----------------------------------------------------------------  }
 TCallingPartyNumber = class(TStruct)
   public
   TypeOfNumber     : byte;        {0x00}
   ScreenIndicator  : byte;        {0x80}
   Contents         : Ansistring;      { calling party number}
   constructor Create(aTypeOfNumber    : byte;
                      aScreenIndicator : byte;
                      aContents        : Ansistring);
   constructor CreateFromData(aDataPtr :pointer);
 end;
{ -----------------------------------------------------------------  }
 TCallingPartySubAddress = class(TStruct)
   public
   TypeOfSubAddress : byte;        {0x80}
   Contents         : Ansistring;      {0x50}
   constructor Create(aTypeOfSubAddress  : byte;
                      aContents          : Ansistring);
   constructor CreateFromData(aDataPtr :pointer);
 end;
{ -----------------------------------------------------------------  }
 TCalledPartyNumber     = TCallingPartySubAddress;
 TCalledPartySubAddress = TCallingPartySubAddress;
 TConnectedNumber       = TCallingPartyNumber;
 TConnectedSubAddress   = TCallingPartySubAddress;
{ -----------------------------------------------------------------  }
 TBProtocol = class(TStruct)
   public
     B1Protocol    : word;
     B2Protocol    : word;
     B3Protocol    : word;
     B1Configuration : TStruct;
     B2Configuration : TStruct;
     B3Configuration : TStruct;
     constructor Create(aB1Protocol    : word;
                        aB2Protocol    : word;
                        aB3Protocol    : word;
                        aB1Configuration : TStruct;
                        aB2Configuration : TStruct;
                        aB3Configuration : TStruct);
     constructor CreateFromData(aDataPtr :pointer);
     destructor Destroy; override;
 end;
{ -----------------------------------------------------------------  }
{ DTMF-support                                                       }
{ -----------------------------------------------------------------  }
TCapiString    = class(TStruct)
  public
    Contents : Ansistring;
    constructor Create(aContents: Ansistring);
    constructor CreateFromData(aDataPtr: Pointer);
end;
TDTMFDigits      = TCapiString;
TDTMFFacilityReqParameter = class(TStruct)
  public
    constructor Create(
         Command      : word;
         ToneDuration : word;
         GapDuration  : word;
         Digits       : TDTMFDigits);
end;
TDTMFFacilityConfParameter = class(TStruct)
  public
    Information : word;
    constructor CreateFromData(aDataPtr: Pointer);
end;
TDTMFFacilityIndParameter = class(TStruct)
  public
    Digits : Ansistring;
    constructor CreateFromData(aDataPtr: Pointer);
end;
TDTMFFacilityRespParameter = class(TStruct)
  constructor Create;
end;
{ -----------------------------------------------------------------  }
{ Support for Supplementary Services                                 }
{ -----------------------------------------------------------------  }
TSuppServFacilityReqParameter = class(TStruct)
  public
    Command         : word;                             { = Function }
    ServiceSpecificParam : TStruct;
    constructor Create_GetSupportedServices;
end;
TSuppServFacilityConfParameter = class(TStruct)
  public
    Command         : word;                             { = Function }
    ServiceSpecificParam : TStruct;
    constructor CreateFromData(aDataPtr: Pointer);
    destructor Destroy; override;
    { the following function(s) need CreateFromData to be called first }
    function GetSupportedServices(var ServiceInfo : word; var SupportedServices : dword) : boolean;
end;
TSuppServFacilityIndParameter = class(TStruct)
  public
    Command         : word;                             { = Function }
    ServiceSpecificParam : TStruct;
    constructor CreateFromData(aDataPtr: Pointer);
    destructor Destroy; override;    
end;
TSuppServFacilityRespParameter = class(TStruct)
  constructor Create;
end;
{ -----------------------------------------------------------------  }
{ This two types are private to this unit and CpMsg.pas
  the user of CpStruct or CpMsg doesn't need them                    }
RCapiMessageHeader = record
  TotalLength   : word;
  ApplId        : word;
  Command       : byte;
  SubCommand    : byte;
  MessageNumber : word;
end;
RStruct = record
  ShortLength  : byte;
  LongLength   : word;
end;
implementation
{ ------------------------------------------------------------------ }
{  TStruct.Create
   If bFreeStruct is true FStruct is released at destruction time.
   Releasing FStruct is only necessary (and allowed) if FStruct is
   not created via CreateFromData. CreateFormData sets the pointer
   to a memory area managed by CAPI.                                 }
{ ------------------------------------------------------------------ }
constructor TStruct.Create;
begin
  inherited Create;
  bFreeStruct := true;
  FBytesRead := 0;
end;
{ ------------------------------------------------------------------ }
{  TStruct.CreateEmpty
   Creates an Empty struct. An empty CAPI struct ist a one byte field
   with value 0                                                      }
{ ------------------------------------------------------------------ }
constructor TStruct.CreateEmpty;
begin
  inherited Create;
  MakeStruct(0);
  bFreeStruct := true;
  FBytesRead := 0;
end;
{ ------------------------------------------------------------------ }
{  TStruct.CreateFromData
   Sets a pointer to the CAPI message (FStruct) and another one
   to the data area of the CAPI message (FStructPtr).
   After creation of TStruct the TStruct children can retrieve
   their data via GetStructData.                                     }
{ ------------------------------------------------------------------ }
constructor TStruct.CreateFromData(aDataPtr : pointer);
begin
  inherited Create;
  bFreeStruct := false;
  FStruct     := aDataPtr;
  { sets FStructPtr to the first byte of the data area }
  {
  FBytesRead  := 0;
  FStructPtr  := FStruct;
  if GetDatalength < 255 then begin
    if GetDatalength > 0 then inc(FStructPtr, SizeOf(byte))
                         else FStructPtr := nil;
  end else begin
    inc(FStructPtr, SizeOf(byte) + SizeOf(word));
  end;
  }
  Reset;        { sets FStructPtr to the first byte of the data area }
end;
{ ------------------------------------------------------------------ }
{  TStruct.Destroy
   Frees the allocated memory for the CAPI message if the message
   was created by the app. and not by CAPI (bFreeStruct = true).
   If CAPI created the message FStruct is a pointer to the memory
   area of CAPI and CAPI frees the memory itself.                    }
{ ------------------------------------------------------------------ }
destructor TStruct.Destroy;
begin
  FStructPtr := nil;
  if bFreeStruct then FreeMem(FStruct, GetLength);
  inherited Destroy;
end;
{ ------------------------------------------------------------------ }
{  TStruct.GetDataLength
   Length of the data area of a struct. Length can be 0.             }
{ ------------------------------------------------------------------ }
Function  TStruct.GetDataLength : word;
var pData    : ^RStruct;
begin
  pData := pointer(FStruct);
  if pData^.ShortLength < 255
     then GetDataLength := pData^.ShortLength
     else GetDataLength := pData^.LongLength;
end;
{ ------------------------------------------------------------------ }
{  TStruct.GetLength
   Length of the data area of the struct + leading length byte(s).
   Min. length is 1 byte if DataLength is 0.                         }
{ ------------------------------------------------------------------ }
Function  TStruct.GetLength : word;
var aDataLength : word;
begin
  aDataLength := GetDataLength;
  if aDataLength < 255
    then GetLength := SizeOf(byte) + aDataLength
    else GetLength := SizeOf(byte) + SizeOf(word) + aDataLength;
end;
{ ------------------------------------------------------------------ }
{  TStruct.MakeStruct
   The children of TStruct use MakeStruct to create a struct with
   fixed length. After creating the struct they use AddStructData
   to insert their data into the structure.                          }
{ ------------------------------------------------------------------ }
Procedure TStruct.MakeStruct(aDataLength : word);
var TotalLength : word;
    pData    : ^RStruct;
begin
  If aDataLength < 255
    then TotalLength :=  SizeOf(byte) + aDataLength
    else TotalLength :=  SizeOf(byte) + SizeOf(word) + aDataLength;
  GetMem(FStruct, TotalLength); { allocate memory for FStruct ... }
  { set pData to FStruct to be easily able to set the length }
  pData      := pointer(FStruct);
  FStructPtr := FStruct;
  If aDataLength < 255 then begin
    pData^.ShortLength := byte(aDataLength);
    { set FStructPtr to the data area - see AddStructData }
    if aDataLength > 0 then inc(FStructPtr, SizeOf(byte))
                       else FStructPtr := nil;
  end else begin
    pData^.ShortLength := 255;
    pData^.LongLength := aDataLength;
    { set FStructPtr to the data area - see AddStructData}
    inc(FStructPtr, SizeOf(byte) + SizeOf(word));
  end;
end;
{ ------------------------------------------------------------------ }
{  TStruct.AddStructData
   Copies data into the memory area allocated within MakeStruct.
   AddStructData must be called in the same order, the data is
   located in the data area.
   AddStructData adds data according to the CAPI struct definition.
   The child object has to take care itself not to add more data
   than it reserved in MakeStructData.                               }
{ ------------------------------------------------------------------ }
Procedure TStruct.AddStructData(aDataPtr : pointer; aDataLength : word);
begin
  { copy aDataLength data to the recent data pointer }
  if aDataLength > 0 then begin
    Move(aDataPtr^, FStructPtr^, aDataLength);
    inc(FStructPtr, aDataLength);
  end;
end;
{ ------------------------------------------------------------------ }
{  TStruct.GetStructData
   Retrieve data from a given data structure (a message sent by CAPI).
   Data in aDataLength is copied to parameter aDataPtr.
   GetStructData must be called in the same order the message
   parameters are located in the data structure from CAPI.
   The Reset method can be used to restart reading from the begin
   of the structure                                                  }  
{ ------------------------------------------------------------------ }
Procedure TStruct.GetStructData(aDataPtr : pointer; aDataLength : word);
begin
  if not ((FBytesRead + aDataLength) > GetDataLength) then begin
    Move(FStructPtr^, aDataPtr^, aDataLength);
    inc(FStructPtr, aDataLength);
    inc(FBytesRead, aDataLength);
  end else begin
    FillChar(aDataPtr^, aDataLength, #0);
  end;
end;
{ ------------------------------------------------------------------ }
{ TStruct.Reset
  Set FStructPtr back to the first byte of the data structre.
  This method can be used to restart reading a data structure via
  GetStructData.                                                     }
{ ------------------------------------------------------------------ }
Procedure TStruct.Reset;
begin
 FStructPtr := FStruct;
 { Zeiger auf den Beginn des Datenbereichs setzen }
 if GetDatalength < 255 then begin
   if GetDatalength > 0 then inc(FStructPtr, SizeOf(byte))
                        else FStructPtr := nil;
 end else begin
   inc(FStructPtr, SizeOf(byte) + SizeOf(word));
 end;
 FBytesRead := 0;
end;
{ ------------------------------------------------------------------ }
{  TCallingPartyNumber.Create
   The app. calls this constructor to add a CallingPartyNumber to
   a CAPI message.
   !!! The public variables (i.e. TypeOfNumber) are not set here.
       We assume that the app. doesn't need them after creation time.
       Apart from that the messsage is destroyed after beeing sent
       to CAPI (see CpHand.pas)                                      }
{ ------------------------------------------------------------------ }
constructor TCallingPartyNumber.Create(aTypeOfNumber    : byte;
                                       aScreenIndicator : byte;
                                       aContents        : Ansistring);
begin
  inherited Create;
  MakeStruct(2 * SizeOf(byte) + System.Length(aContents));
  AddStructData(@aTypeOfNumber, SizeOf(aTypeOfNumber));
  AddStructData(@aScreenIndicator, SizeOf(aScreenIndicator));
  { We have to use System.Length because the compiler would otherwise
    take TStruct.Length instead. What we need here is the length of
    the string }
  AddStructData(pChar(aContents), System.Length(aContents));
end;
{ ------------------------------------------------------------------ }
{  TCallingPartyNumber.CreateFromData
   is called when CAPI sends messages to the app. The public
   variables are set thus the app. can easily use them.              }
{ ------------------------------------------------------------------ }
constructor TCallingPartyNumber.CreateFromData(aDataPtr :pointer);
var ndLen : integer;
begin
  { Sets FData und FStructPtr }
  inherited CreateFromData(aDataPtr);
  GetStructData(@TypeOfNumber, SizeOf(TypeOfNumber));
  GetStructData(@ScreenIndicator, SizeOf(ScreenIndicator));
  ndLen := GetDataLength - 2 * SizeOf(Byte);
  if ndLen > 0 then begin
    SetLength(Contents, ndLen);
    GetStructData(pChar(Contents), ndLen);
  end else Contents := '';
end;
{ ------------------------------------------------------------------ }
{  TCallingPartySubAddress.Create                                    }
{ ------------------------------------------------------------------ }
Constructor TCallingPartySubAddress.Create(aTypeOfSubAddress  : byte;
                                           aContents          : Ansistring);
begin
  inherited Create;
  MakeStruct(SizeOf(byte) + System.Length(aContents));
  AddStructData(@aTypeOfSubAddress, SizeOf(byte));
  AddStructData(pChar(aContents), System.Length(aContents));
end;
{ ------------------------------------------------------------------ }
{  TCallingPartySubAddress.CreateFromData                            }
{ ------------------------------------------------------------------ }
Constructor TCallingPartySubAddress.CreateFromData(aDataPtr :pointer);
var ndLen : integer;
begin
  { Sets FData und FStructPtr }
  inherited CreateFromData(aDataPtr);
  GetStructData(@TypeOfSubAddress, SizeOf(TypeOfSubAddress));
  ndLen := GetDataLength - SizeOf(TypeOfSubAddress);
  if ndLen > 0 then begin
    SetLength(Contents, ndLen);
    GetStructData(pChar(Contents), ndLen);
  end else Contents := '';
end;
{ ------------------------------------------------------------------ }
{  TBProtocol.Create                                                 }
{ ------------------------------------------------------------------ }
Constructor TBProtocol.Create(aB1Protocol    : word;
                              aB2Protocol    : word;
                              aB3Protocol    : word;
                              aB1Configuration : TStruct;
                              aB2Configuration : TStruct;
                              aB3Configuration : TStruct);
begin
  inherited Create;
  MakeStruct(3 * SizeOf(word) +
             aB1Configuration.Length +
             aB2Configuration.Length +
             aB3Configuration.Length);
  AddStructData(@aB1Protocol, SizeOf(aB1Protocol));
  AddStructData(@aB2Protocol, SizeOf(aB2Protocol));
  AddStructData(@aB3Protocol, SizeOf(aB3Protocol));
  AddStructData(aB1Configuration.Header, aB1Configuration.Length);
  AddStructData(aB2Configuration.Header, aB2Configuration.Length);
  AddStructData(aB3Configuration.Header, aB3Configuration.Length);
  { free substructurs, data is stored in FStruct }
  aB1Configuration.Free;
  aB2Configuration.Free;
  aB3Configuration.Free;
end;
{ ------------------------------------------------------------------ }
{  TBProtocol.CreateFromData                                         }
{ ------------------------------------------------------------------ }
Constructor TBProtocol.CreateFromData(aDataPtr :pointer);
begin
  { Sets FData und FStructPtr }
  inherited CreateFromData(aDataPtr);
  GetStructData(@B1Protocol, SizeOf(B1Protocol));
  GetStructData(@B2Protocol, SizeOf(B2Protocol));
  GetStructData(@B3Protocol, SizeOf(B3Protocol));
  { B1Configuration.Header is the pointer to the first byte of
    the struct B1Configuration }
  B1Configuration := TStruct.CreateFromData(FStructPtr);
    inc(FStructPtr, B1Configuration.Length);
  B2Configuration := TStruct.CreateFromData(FStructPtr);
    inc(FStructPtr, B2Configuration.Length);
  B3Configuration := TStruct.CreateFromData(FStructPtr);
    inc(FStructPtr, B3Configuration.Length);
end;
{ ------------------------------------------------------------------ }
{  TBProtocol.Destroy                                                }
{ ------------------------------------------------------------------ }
destructor TBProtocol.Destroy;
begin
  B1Configuration.Free;
  B2Configuration.Free;
  B3Configuration.Free;
  inherited Destroy;
end;
{ ------------------------------------------------------------------ }
{  TCapiString.Create                                                }
{ ------------------------------------------------------------------ }
constructor TCapiString.Create(aContents: Ansistring);
begin
  inherited create;
  MakeStruct(System.Length(aContents));
  AddStructData(pChar(aContents), System.Length(aContents));
end;
{ ------------------------------------------------------------------ }
{  TCapiString.CreateFromData                                        }
{ ------------------------------------------------------------------ }
constructor TCapiString.CreateFromData(aDataPtr: Pointer);
begin
  inherited CreateFromData(aDataPtr);
  if GetDataLength > 0 then begin
    SetLength(Contents, GetDataLength);
    GetStructData(pChar(Contents), GetDataLength);
  end else Contents := '';
end;
{ ------------------------------------------------------------------ }
{  TDTMFFacilityReqParameter.Create                                  }
{ ------------------------------------------------------------------ }
constructor TDTMFFacilityReqParameter.Create(Command : word;
      ToneDuration : word; GapDuration  : word; Digits : TDTMFDigits);
begin
  inherited create;
  MakeStruct(3*SizeOf(Word) + Digits.Length);
  AddStructData(@Command, SizeOf(Command));
  AddStructData(@ToneDuration, SizeOf(ToneDuration));
  AddStructData(@GapDuration, SizeOf(GapDuration));
  AddStructData(Digits.Header, Digits.Length);
  Digits.Free;
end;
{ ------------------------------------------------------------------ }
{  TDTMFFacilityConfParameter.CreateFromData                         }
{ ------------------------------------------------------------------ }
constructor TDTMFFacilityConfParameter.CreateFromData(aDataPtr: Pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetStructData(@Information, SizeOf(Information));
end;
{ ------------------------------------------------------------------ }
{  TDTMFFacilityIndParameter.CreateFromData                          }
{ ------------------------------------------------------------------ }
constructor TDTMFFacilityIndParameter.CreateFromData(aDataPtr: Pointer);
begin
  inherited CreateFromData(aDataPtr);
  if GetDataLength > 0 then
  begin
    SetLength(Digits, GetDataLength);
    GetStructData(pChar(Digits), GetDataLength);
  end else Digits := '';
end;
{ ------------------------------------------------------------------ }
{ TDTMFFacilityRespParameter.Create                                  }
{ ------------------------------------------------------------------ }
constructor TDTMFFacilityRespParameter.Create;
begin
  inherited CreateEmpty;
end;
{ ------------------------------------------------------------------ }
{  TSuppServFacilityReqParameter.Create_GetSupportedServices         }
{ ------------------------------------------------------------------ }
constructor TSuppServFacilityReqParameter.Create_GetSupportedServices;
begin
  inherited Create;
  ServiceSpecificParam := TStruct.CreateEmpty;
  MakeStruct(SizeOf(word) + ServiceSpecificParam.Length);
  AddStructData(@cSSGetSupportedServices, SizeOf(word));
  AddStructData(ServiceSpecificParam.Header, ServiceSpecificParam.Length);
  ServiceSpecificParam.Free;
end;
{ ------------------------------------------------------------------ }
{  TSuppServFacilityConfParameter.CreateFromData                     }
{ ------------------------------------------------------------------ }
constructor TSuppServFacilityConfParameter.CreateFromData(aDataPtr: Pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetStructData(@Command, SizeOf(Command));
  ServiceSpecificParam := TStruct.CreateFromData(FStructPtr);
  inc(FStructPtr, ServiceSpecificParam.Length);
end;
{ ------------------------------------------------------------------ }
{  TSuppServFacilityConfParameter.Destroy                            }
{ ------------------------------------------------------------------ }
destructor TSuppServFacilityConfParameter.Destroy;
begin
  ServiceSpecificParam.Free;
  inherited destroy;
end;
{ ------------------------------------------------------------------ }
{  TSuppServFacilityConfParameter.GetSupportedServices
   siehe CpConst.CapiGetSupportedServicesString                      }
{ ------------------------------------------------------------------ }
function TSuppServFacilityConfParameter.GetSupportedServices(var ServiceInfo : word; var SupportedServices : dword) : boolean;
begin
  if Command <> cSSGetSupportedServices then begin
    Result := false;
    exit;
  end;
  ServiceSpecificParam.GetStructData(@ServiceInfo, SizeOf(ServiceInfo));
  ServiceSpecificParam.GetStructData(@SupportedServices, SizeOf(SupportedServices));
  { !!! Reset the pointer to enable a second call to this function }
  ServiceSpecificParam.Reset;
  Result := true;
end;
{ ------------------------------------------------------------------ }
{  TSuppServFacilityIndParameter.CreateFromData                      }
{ ------------------------------------------------------------------ }
constructor TSuppServFacilityIndParameter.CreateFromData(aDataPtr: Pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetStructData(@Command, SizeOf(Command));
  ServiceSpecificParam := TStruct.CreateFromData(FStructPtr);
  inc(FStructPtr, ServiceSpecificParam.Length);
end;
{ ------------------------------------------------------------------ }
{  TSuppServFacilityInfParameter.Destroy                             }
{ ------------------------------------------------------------------ }
destructor TSuppServFacilityIndParameter.Destroy;
begin
  ServiceSpecificParam.Free;
  inherited destroy;
end;
{ ------------------------------------------------------------------ }
{ TSuppServFacilityRespParameter.Create                              }
{ ------------------------------------------------------------------ }
constructor TSuppServFacilityRespParameter.Create;
begin
  inherited CreateEmpty;
end;
end.

