{ -----------------------------------------------------------------  }
{ CpMsg
  Author:              Peter Zwosta
  Homepage: http://home.t-online.de/home/Peter.Zwosta                }
{ -----------------------------------------------------------------  }
{ CpMsg declares and implements the Capi messages and provides two
  functions (CapiCreateMessage, CapiFreeMessage) to create and
  release the CAPI messages.                                         }
{ -------------------------------------------------------------------}
{ History:
  Oct 20, 1999 pz added samples f. InfoInd
  Feb 16, 1999 pz added samples f. FacilityReq
  Nov 29, 1998 pz removed TypeCast from CapiFreeMessage, added
       TConnectB3T90ActiveXXX, ResetB3XXX, SelectBProtocolXXX
  Nov 20, 1998 pz, rb (Rdiger Bund) - added FacilityXXX,
       InfoReq and InfoConf.
  Nov 7, 1998 pz - history inserted :-))))                           }
{ -----------------------------------------------------------------  }
unit CpMsg;

interface
uses SysUtils, Dialogs, Windows, CpConst, CpStruct;

type
{ -----------------------------------------------------------------  }
{ TCapiMessageHeader
  Length :      length of the header
  TotalLength : length of the whole CAPI message including the
                word for TotalLength
                A CapiMessage is not a TStruct as in a TStruct
                length is the length of the following data structure
                without the first word.                              }
{ -----------------------------------------------------------------  }
 TCapiMessageHeader = class(TObject)
    procedure SetWordCommand(aWordCommand : word); virtual;
    function  GetWordCommand : word; virtual;
    function  GetLength : word;
   public
    TotalLength   : word;
    ApplId        : word;
    Command       : byte;
    SubCommand    : byte;
    MessageNumber : word;
    constructor Create;
    constructor CreateFromData(aData : pointer);

    property WordCommand : word read GetWordCommand write SetWordCommand;
    property Length  : word read GetLength;
 end;

{ -----------------------------------------------------------------  }
{ TCapiMessage
  This is the parent of all CapiMessage object.
  MessagePtr is used by the app. to get a pointer to the Message.
  This pointer is passed to CAPI (Capi_PutMessage).
  Create, MakeData und AddData  -    used for messages to CAPI,
  CreateFromData, und GetData   -    used for messages from CAPI
{ -----------------------------------------------------------------  }
TCapiMessage = class(TObject)
   FData       : pByte;
   FDataPtr    : pByte;
   Function GetMessagePtr : pointer;
  public
   Header     : TCapiMessageHeader;
   constructor Create;
   constructor CreateFromData(aDataPtr : pointer); virtual;
   Destructor Destroy; override;
   Procedure MakeData(aMsgLength : word);
   Procedure AddData(aDataPtr : pointer; aDataLength : word);
   Procedure GetData(aDataPtr : pointer; aDataLength : word);
   property MessagePtr : pointer read GetMessagePtr;
end;

{ -----------------------------------------------------------------  }
{ These function are used to Create and release TCapiMessages        }
{ -----------------------------------------------------------------  }
function  CapiCreateMessage(aCMsg : pointer) : TCapiMessage;
procedure CapiFreeMessage(var aCapiMessage : TCapiMessage);

type
{ -----------------------------------------------------------------  }
{ TCapiConfirmation                                                  }
{ -----------------------------------------------------------------  }
TCapiConfirmation = class(TCapiMessage)
  public
   Controller_PLCI_NCCI : dWord;
   Info                 : Word;
   constructor CreateFromData(aDataPtr : pointer); override;
end;
{ -----------------------------------------------------------------  }
{ LISTEN                                                             }
{ -----------------------------------------------------------------  }
TListenReq = class(TCapiMessage)
  public
   constructor Create(
          Controller             : dWord;
          InfoMask               : dWord;
          CIPMask                : dWord;
          CIPMask2               : dWord;
          CallingPartyNumber     : TCallingPartyNumber;
          CallingPartySubaddress : TCallingPartySubAddress);
end;
TListenConf = class(TCapiConfirmation)
  public
   property Controller : dWord read Controller_PLCI_NCCI
                               write Controller_PLCI_NCCI;
end;
{ -----------------------------------------------------------------  }
{ CONNECT                                                            }
{ -----------------------------------------------------------------  }
TConnectReq = class(TCapiMessage)
  public
   constructor Create(
          Controller              : dword;
          CipValue                : word;
          CalledPartyNumber       : TCalledPartyNumber;
          CallingPartyNumber      : TCallingPartyNumber;
          CalledPartySubaddress   : TCalledPartySubaddress;
          CallingPartySubaddress  : TCallingPartySubaddress;
          BProtocol               : TBProtocol;
          BC                      : TStruct;
          LLC                     : TStruct;
          HLC                     : TStruct;
          AdditionalInfo          : TStruct);
end;
TConnectConf = class(TCapiConfirmation)
  public
   property PLCI : dWord read Controller_PLCI_NCCI
                         write Controller_PLCI_NCCI;
end;
TConnectInd = class(TCapiMessage)
  public
   PLCI                    : dWord;
   CIPValue                : Word;
   CalledPartyNumber       : TCalledPartyNumber;
   CallingPartyNumber      : TCallingPartyNumber;
   CalledPartySubaddress   : TCalledPartySubaddress;
   CallingPartySubaddress  : TCallingPartySubaddress;
   BC                      : TStruct;
   LLC                     : TStruct;
   HLC                     : TStruct;
   AdditionalInfo          : TStruct;
   constructor CreateFromData(aDataPtr : pointer); override;
   destructor Destroy; override;
end;
TConnectResp = class(TCapiMessage)
  public
   constructor Create(
      PLCI                 : dWord;
      Reject	           : Word;
      BProtocol	           : TBProtocol;
      ConnectedNumber      : TConnectedNumber;
      ConnectedSubaddress  : TConnectedSubaddress;
      LLC                  : TStruct;
      AdditionalInfo       : TStruct);
end;
{ -----------------------------------------------------------------  }
{ CONNECT_ACTIVE                                                     }
{ -----------------------------------------------------------------  }
TConnectActiveInd = class(TCapiMessage)
  public
    PLCI : dword;
    ConnectedNumber : TConnectedNumber;
    ConnectedSubAddress : TConnectedSubAddress;
    LLC : TStruct;
  constructor CreateFromData(aDataPtr : pointer); override;
  destructor Destroy; override;
end;
TConnectActiveResp = class(TCapiMessage)
  public
   constructor Create(
      PLCI                 : dWord);
end;
{ -----------------------------------------------------------------  }
{ CONNECT_B3                                                         }
{ -----------------------------------------------------------------  }
TConnectB3Req = class(TCapiMessage)
  public
    constructor Create(
       PLCI : dword;
       NCPI : TStruct);
end;
TConnectB3Conf = class(TCapiConfirmation)
  public
   property NCCI : dWord read  Controller_PLCI_NCCI
                         write Controller_PLCI_NCCI;
end;
TConnectB3Ind = class(TCapiMessage)
  public
   NCCI  : dWord;
   NCPI  : TStruct;
   constructor CreateFromData(aDataPtr : pointer); override;
   destructor destroy; override;
end;
TConnectB3Resp = class(TCapiMessage)
  public
    constructor Create(
       NCCI   : dword;
       Reject : word;
       NCPI   : TStruct);
end;
{ -----------------------------------------------------------------  }
{ CONNECT_B3_ACTIVE                                                  }
{ -----------------------------------------------------------------  }
TConnectB3ActiveInd = class(TCapiMessage)
  public
   NCCI  : dWord;
   NCPI  : TStruct;
   constructor CreateFromData(aDataPtr : pointer); override;
   destructor destroy; override;
end;
TConnectB3ActiveResp = class(TCapiMessage)
  public
    constructor Create(NCCI   : dword);
end;
{ -----------------------------------------------------------------  }
{ CONNECT_B3_T90_ACTIVE                                              }
{ -----------------------------------------------------------------  }
TConnectB3T90ActiveInd = class(TCapiMessage)
  public
   NCCI  : dWord;
   NCPI  : TStruct;
   constructor CreateFromData(aDataPtr : pointer); override;
   destructor destroy; override;
end;
TConnectB3T90ActiveResp = class(TCapiMessage)
  public
    constructor Create(NCCI   : dword);
end;
{ -----------------------------------------------------------------  }
{ DATA_B3                                                            }
{ -----------------------------------------------------------------  }
TDataB3Req = class(TCapiMessage)
  public
    constructor Create(
       NCCI : dword;
       Data : dword;
       DataLength : word;
       DataHandle : word;
       Flags : word);
end;
TDataB3Conf = class(TCapiMessage)
  public
   NCCI  : dWord;
   DataHandle : word;
   Info  : Word;
   constructor CreateFromData(aDataPtr : pointer); override;
end;
TDataB3Ind = class(TCapiMessage)
  public
   NCCI  : dWord;
   Data  : dword;
   DataLength : word;
   DataHandle : word;
   Flags : word;
   constructor CreateFromData(aDataPtr : pointer); override;
end;
TDataB3Resp = class(TCapiMessage)
  public
    constructor Create(
       NCCI   : dword;
       DataHandle : word);
end;
{ -----------------------------------------------------------------  }
{ DISCONNECT_B3                                                      }
{ -----------------------------------------------------------------  }
TDisconnectB3Req = class(TCapiMessage)
  public
    constructor Create(
       NCCI : dword;
       NCPI : TStruct);
end;
TDisconnectB3Conf = class(TCapiConfirmation)
  public
   property NCCI : dWord read  Controller_PLCI_NCCI
                         write Controller_PLCI_NCCI;
end;
TDisconnectB3Ind = class(TCapiMessage)
  public
   NCCI  : dWord;
   Reason_B3 : word;
   NCPI  : TStruct;
   constructor CreateFromData(aDataPtr : pointer); override;
   destructor destroy; override;
end;
TDisconnectB3Resp = class(TCapiMessage)
  public
    constructor Create(
       NCCI   : dword);
end;
{ -----------------------------------------------------------------  }
{ DISCONNECT                                                         }
{ -----------------------------------------------------------------  }
TDisconnectReq  = class(TCapiMessage)
  public
   constructor Create(PLCI : dWord;
                      AdditionalInfo : TStruct);
end;
TDisconnectConf = class(TCapiConfirmation)
  public
   property PLCI : dWord read  Controller_PLCI_NCCI
                         write Controller_PLCI_NCCI;
end;
TDisconnectInd = class(TCapiMessage)
  public
   PLCI                    : dWord;
   Reason                  : word;
   constructor CreateFromData(aDataPtr : pointer); override;
end;
TDisconnectResp = class(TCapiMessage)
  public
   constructor Create(PLCI : dWord);
end;
{ -----------------------------------------------------------------  }
{ INFO                                                               }
{ -----------------------------------------------------------------  }
TInfoReq = class(TCapiMessage)
  public
    constructor Create(
       PLCI              : dword;
       CalledPartyNumber : TCalledPartyNumber;
       AdditionalInfo    : TStruct);
end;
TInfoConf = class(TCapiConfirmation)
  public
   property PLCI : dWord read  Controller_PLCI_NCCI
                         write Controller_PLCI_NCCI;
end;
TInfoInd = class(TCapiMessage)
  public
   PLCI        : dWord;
   InfoNumber  : word;
   InfoElement : TStruct;
   constructor CreateFromData(aDataPtr : pointer); override;
   destructor Destroy; override;
end;
TInfoResp = class(TCapiMessage)
  public
   constructor Create(PLCI : dWord);
end;
{ -----------------------------------------------------------------  }
{ ALERT                                                              }
{ -----------------------------------------------------------------  }
TAlertReq = class(TCapiMessage)
  public
   constructor Create(
     PLCI : dword;
     AdditionalInfo : TStruct);
end;
TAlertConf = class(TCapiConfirmation)
  public
   property PLCI : dWord read  Controller_PLCI_NCCI
                         write Controller_PLCI_NCCI;
end;
{ -----------------------------------------------------------------  }
{ FACILITY                                                           }
{ -----------------------------------------------------------------  }
TFacilityReq = class(TCapiMessage)
  public
    constructor Create(Controller_PLCI_NCCI : dWord;
                       FacilitySelector : word;
                       FacilityParameters : TStruct);
end;
TFacilityConf = class(TCapiMessage)
  public
    Controller_PLCI_NCCI : dWord;
    Info                 : word;
    FacilitySelector     : word;
    FacilityParameters   : TStruct;

    constructor CreateFromData(aDataPtr : pointer); override;
    destructor  Destroy; override;

    property Controller : dWord read Controller_PLCI_NCCI write Controller_PLCI_NCCI;
    property PLCI : dWord read Controller_PLCI_NCCI write Controller_PLCI_NCCI;
    property NCCI : dWord read Controller_PLCI_NCCI write Controller_PLCI_NCCI;
end;
TFacilityInd  = class(TCapiMessage)
  public
    Controller_PLCI_NCCI : dWord;
    FacilitySelector     : word;
    FacilityParameters   : TStruct;

    constructor CreateFromData(aDataPtr : pointer); override;
    destructor  Destroy; override;

    property Controller : dWord read Controller_PLCI_NCCI write Controller_PLCI_NCCI;
    property PLCI : dWord read Controller_PLCI_NCCI write Controller_PLCI_NCCI;
    property NNCI : dWord read Controller_PLCI_NCCI write Controller_PLCI_NCCI;
end;
TFacilityResp = class(TCapiMessage)
  public
    constructor Create(Controller_PLCI_NCCI : dWord;
                       FacilitySelector : word;
                       FacilityParameters : TStruct);
end;
{ -----------------------------------------------------------------  }
{ RESET_B3                                                           }
{ -----------------------------------------------------------------  }
TResetB3Req = class(TCapiMessage)
  public
    constructor Create(
       NCCI : dword;
       NCPI : TStruct);
end;
TResetB3Conf = class(TCapiConfirmation)
  public
   property NCCI : dWord read  Controller_PLCI_NCCI
                         write Controller_PLCI_NCCI;
end;
TResetB3Ind = class(TCapiMessage)
  public
   NCCI  : dWord;
   NCPI  : TStruct;
   constructor CreateFromData(aDataPtr : pointer); override;
   destructor destroy; override;
end;
TResetB3Resp = class(TCapiMessage)
  public
    constructor Create(NCCI   : dword);
end;
{ -----------------------------------------------------------------  }
{ SELECT_B_PROTOCOL                                                  }
{ -----------------------------------------------------------------  }
TSelectBProtocolReq = class(TCapiMessage)
  public
    constructor Create(PLCI      : dword;
                       BProtocol : TBProtocol);
end;
TSelectBProtocolConf = class(TCapiConfirmation)
  public
   property PLCI : dWord read  Controller_PLCI_NCCI
                         write Controller_PLCI_NCCI;
end;

implementation { ****************************************************}
{ ------------------------------------------------------------------ }
{ TCapiMessageHeader.Create
  sets deaults for Header values                                     }
{ ------------------------------------------------------------------ }
constructor TCapiMessageHeader.Create;
begin
  inherited Create;
  MessageNumber := 0; { 0 is default, the app. can use the
                        MessageNumber for own purposes }
end;
{ ------------------------------------------------------------------ }
{ TCapiMessageHeader.CreateFromData
  fills the public variables using an 'unstructured' pointer to a
  message from CAPI. Unstructured of course means no Delphi
  structures.                                                        }
{ ------------------------------------------------------------------ }
constructor TCapiMessageHeader.CreateFromData(aData : pointer);
var DataRec : ^RCapiMessageHeader;
begin
  inherited create;
  DataRec       := aData;
  Totallength   := DataRec^.Totallength;
  ApplID        := DataRec^.ApplID;
  Command       := DataRec^.Command;
  SubCommand    := DataRec^.SubCommand;
  MessageNumber := DataRec^.MessageNumber;
end;
{ ------------------------------------------------------------------ }
{ TCapiMessageHeader.SetWordCommand u. GetWordCommand
  Enables usage of consts like LISTEN_REQ (see CpConst.pas) to set
  Command and SubCommand in one instruction.                         }
{ ------------------------------------------------------------------ }
procedure TCapiMessageHeader.SetWordCommand(aWordCommand : word);
begin
  Command     := byte(aWordCommand shr 8);
  SubCommand  := byte(aWordCommand);
end;
function  TCapiMessageHeader.GetWordCommand : word;
begin
  GetWordCommand := Command shl 8 + SubCommand;
end;
{ ------------------------------------------------------------------ }
{ TCapiMessageHeader.GetLength
  length of the header. This length is used by TCapiMessage objects
  to reserve the correct amount of memory for the message
  (see TListenREQ.Create).                                           }
{ ------------------------------------------------------------------ }
function  TCapiMessageHeader.GetLength : word;
begin
  GetLength := SizeOf(RCapiMessageHeader);
end;
{ ------------------------------------------------------------------ }
{ TCapiMessage.Create                                                }
{ ------------------------------------------------------------------ }
constructor TCapiMessage.Create;
begin
  inherited Create;
  Header := TCapiMessageHeader.Create;
  FData := nil;
  FDataPtr := nil;
end;
{ ------------------------------------------------------------------ }
{ TCapiMessage.CreateFromData                                        }
{ ------------------------------------------------------------------ }
constructor TCapiMessage.CreateFromData(aDataPtr : pointer);
begin
  inherited Create;
  { Create Header }
  Header := TCapiMessageHeader.CreateFromData(aDataPtr);
  { Move data from memory area of CAPI to own memory area }
  GetMem(FData, Header.TotalLength);
  Move(aDataPtr^, FData^, Header.TotalLength);
  FDataPtr := FData;
  { FDataPtr points to the data area at the end of the header
    thus the message objects can retrieve their data }
  inc(FDataPtr, Header.Length);
end;
{ ------------------------------------------------------------------ }
{ TCapiMessage.MakeData                                              }
{ ------------------------------------------------------------------ }
Procedure TCapiMessage.MakeData(aMsgLength : word);
begin
  GetMem(FData, aMsgLength);        { allocate memory for FData      }
  FDataPtr := FData;
  { the header is copied later in GetMessagePtr. FDataPtr is set
    behind the header so the message objects can copy their data into
    memory area for CAPI }
  inc(FDataPtr, Header.Length);
end;
{ ------------------------------------------------------------------ }
{ TCapiMessage.AddData
  There's no code testing whether the message object copies more data
  than there is memory allocated. The message object has to take
  care itself.                                                       }
{ ------------------------------------------------------------------ }
Procedure TCapiMessage.AddData(aDataPtr : pointer; aDataLength : word);
begin
  { copy aDataLength Bytes of data to FDataPtr }
  if aDataLength > 0 then begin
    Move(aDataPtr^, FDataPtr^, aDataLength);
    inc(FDataPtr, aDataLength);
  end;
end;
{ ------------------------------------------------------------------ }
{ TCapiMessage.GetData                                               }
{ ------------------------------------------------------------------ }
Procedure TCapiMessage.GetData(aDataPtr : pointer; aDataLength : word);
begin
  Move(FDataPtr^, aDataPtr^, aDataLength);
  inc(FDataPtr, aDataLength);
end;
{ ------------------------------------------------------------------ }
{ TCapiMessage.GetMessagePtr                                         }
{ ------------------------------------------------------------------ }
Function TCapiMessage.GetMessagePtr : pointer;
var cHeader : RCapiMessageHeader;
begin
  { copy header to the structure }
  FDataPtr := FData;
  cHeader.TotalLength   := Header.TotalLength;
  cHeader.ApplID        := Header.ApplID;
  cHeader.Command       := Header.Command;
  cHeader.SubCommand    := Header.SubCommand;
  cHeader.MessageNumber := Header.MessageNumber;
  Move(cHeader, FDataPtr^, SizeOf(cHeader));
  GetMessagePtr := pointer(FData);
end;
{ ------------------------------------------------------------------ }
{ TCapiMessage.Destroy                                               }
{ ------------------------------------------------------------------ }
destructor TCapiMessage.Destroy;
begin
  { in Create FData is always created, no means if the message
    is from CAPI or from the app. to CAPI } 
  If FData <> nil then FreeMem(FData, Header.TotalLength);
  Header.Free;
  inherited Destroy;
end;
{  ------------------------------------------------------------------}
{ TCapiConfirmation                                                  }
{ ------------------------------------------------------------------ }
constructor TCapiConfirmation.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@Controller_PLCI_NCCI, SizeOf(Controller_PLCI_NCCI));
  GetData(@Info, SizeOf(Info));
end;
{ -------------------------------------------------------------------}
{ TListenREQ                                                         }
{ -------------------------------------------------------------------}
constructor TListenREQ.Create(Controller             : dWord;
                              InfoMask               : dWord;
                              CIPMask                : dWord;
                              CIPMask2               : dWord;
                              CallingPartyNumber     : TCallingPartyNumber;
                              CallingPartySubaddress : TCallingPartySubAddress);

var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +                    { calculate length }
                4 * SizeOf(dWord) +
                CallingPartyNumber.Length +
                CallingPartySubaddress.Length;
  MakeData(MsgLength);                   { allocate memory for FData }
  { ApplId and MessageNumber is set by the app. }
  with Header do begin
    TotalLength := MsgLength;
    WordCommand := LISTEN_REQ;
  end;
  { fill FData with data from substructures }
  AddData(@Controller, SizeOf(dWord));
  AddData(@InfoMask, SizeOf(dWord));
  AddData(@CIPMask, SizeOf(dWord));
  AddData(@CIPMask2, SizeOf(dWord));
  AddData(CallingPartyNumber.Header, CallingPartyNumber.Length);
  AddData(CallingPartySubaddress.Header, CallingPartySubaddress.Length);
  { after the data is copied to FData the sub structures can be
    released. We assume that the app. doesn't use these structures
    after having sent the message }
  CallingPartyNumber.Free;
  CallingPartySubaddress.Free;
end;
{  ------------------------------------------------------------------}
{ TConnectInd                                                        }
{ ------------------------------------------------------------------ }
constructor TConnectInd.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@PLCI, SizeOf(PLCI));
  GetData(@CIPValue, SizeOf(CIPValue));
  CalledPartyNumber := TCalledPartyNumber.CreateFromData(FDataPtr);
    { move pointer thus the next structure can retrieve its data
      when using GetData the data pointer is moved automatically }
    inc(FDataPtr, CalledPartyNumber.Length);
  CallingPartyNumber := TCallingPartyNumber.CreateFromData(FDataPtr);
    inc(FDataPtr, CallingPartyNumber.Length);
  CalledPartySubaddress := TCalledPartySubaddress.CreateFromData(FDataPtr);
    inc(FDataPtr, CalledPartySubaddress.Length);
  CallingPartySubaddress := TCallingPartySubaddress.CreateFromData(FDataPtr);
    inc(FDataPtr, CallingPartySubaddress.Length);
  BC := TStruct.CreateFromData(FDataPtr);
    inc(FDataPtr, BC.Length);
  LLC := TStruct.CreateFromData(FDataPtr);
    inc(FDataPtr, LLC.Length);
  HLC := TStruct.CreateFromData(FDataPtr);
    inc(FDataPtr, HLC.Length);
  AdditionalInfo := TStruct.CreateFromData(FDataPtr);
    inc(FDataPtr, AdditionalInfo.Length);
end;
destructor TConnectInd.Destroy;
begin
  { release structures }
  CalledPartyNumber.Free;
  CallingPartyNumber.Free;
  CalledPartySubaddress.Free;
  CallingPartySubaddress.Free;
  BC.Free;
  LLC.Free;
  HLC.Free;
  AdditionalInfo.Free;
  inherited Destroy;
end;
{ -------------------------------------------------------------------}
{ TConnectReq                                                        }
{ ------------------------------------------------------------------ }
constructor TConnectReq.Create(
          Controller              : dword;
          CipValue                : word;
          CalledPartyNumber       : TCalledPartyNumber;
          CallingPartyNumber      : TCallingPartyNumber;
          CalledPartySubaddress   : TCalledPartySubaddress;
          CallingPartySubaddress  : TCallingPartySubaddress;
          BProtocol               : TBProtocol;
          BC                      : TStruct;
          LLC                     : TStruct;
          HLC                     : TStruct;
          AdditionalInfo          : TStruct);

var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +                    { calculate length }
                SizeOf(dWord) +
                SizeOf(word) +
                CalledPartyNumber.Length +
                CallingPartyNumber.Length +
                CalledPartySubaddress.Length +
                CallingPartySubaddress.Length +
                BProtocol.Length +
                BC.Length +
                LLC.Length +
                HLC.Length +
                AdditionalInfo.Length;
  MakeData(MsgLength);
  { ApplId and MessageNumber is set by the app. }
  with Header do begin
    TotalLength := MsgLength;
    WordCommand := CONNECT_REQ;
  end;
  { fill FData with data from substructures }
  AddData(@Controller, SizeOf(Controller));
  AddData(@CipValue, SizeOf(CipValue));
  AddData(CalledPartyNumber.Header, CalledPartyNumber.Length);
  AddData(CallingPartyNumber.Header, CallingPartyNumber.Length);
  AddData(CalledPartySubaddress.Header, CalledPartySubaddress.Length);
  AddData(CallingPartySubaddress.Header, CallingPartySubaddress.Length);

  AddData(BProtocol.Header, BProtocol.Length);
  AddData(BC.Header, BC.Length);
  AddData(LLC.Header, LLC.Length);
  AddData(HLC.Header, HLC.Length);
  AddData(AdditionalInfo.Header, AdditionalInfo.Length);
  { after the data is copied to FData the sub structures can be
    released. We assume that the app. doesn't use these structures
    after having sent the message }
  CalledPartyNumber.Free;
  CallingPartyNumber.Free;
  CalledPartySubaddress.Free;
  CallingPartySubaddress.Free;
  BProtocol.Free;
  BC.Free;
  LLC.Free;
  HLC.Free;
  AdditionalInfo.Free;
end;
{ -------------------------------------------------------------------}
{ TConnectResp                                                       }
{ ------------------------------------------------------------------ }
constructor TConnectResp.Create(
       PLCI                 : dWord;
       Reject	            : Word;
       BProtocol	    : TBProtocol;
       ConnectedNumber      : TConnectedNumber;
       ConnectedSubaddress  : TConnectedSubaddress;
       LLC                  : TStruct;
       AdditionalInfo       : TStruct);

var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord) +
                SizeOf(word) +
                BProtocol.Length +
                ConnectedNumber.Length +
                ConnectedSubAddress.Length +
                LLC.Length +
                AdditionalInfo.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := CONNECT_RESP;
  end;

  AddData(@PLCI, SizeOf(PLCI));
  AddData(@Reject, SizeOf(Reject));
  AddData(BProtocol.Header, BProtocol.Length);
  AddData(ConnectedNumber.Header, ConnectedNumber.Length);
  AddData(ConnectedSubAddress.Header, ConnectedSubAddress.Length);
  AddData(LLC.Header, LLC.Length);
  AddData(AdditionalInfo.Header, AdditionalInfo.Length);

  BProtocol.Free;
  ConnectedNumber.Free;
  ConnectedSubAddress.Free;
  LLC.Free;
  AdditionalInfo.Free;
end;
{  ------------------------------------------------------------------}
{ TConnectActiveInd                                                  }
{ ------------------------------------------------------------------ }
constructor TConnectActiveInd.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@PLCI, SizeOf(PLCI));
  ConnectedNumber := TConnectedNumber.CreateFromData(FDataPtr);
    inc(FDataPtr, ConnectedNumber.Length);
  ConnectedSubAddress := TConnectedSubAddress.CreateFromData(FDataPtr);
    inc(FDataPtr, ConnectedSubAddress.Length);
  LLC := TStruct.CreateFromData(FDataPtr);
    inc(FDataPtr, LLC.Length);
end;
destructor TConnectActiveInd.Destroy;
begin
  ConnectedNumber.Free;
  ConnectedSubAddress.Free;
  LLC.Free;
  inherited Destroy;
end;
{ ------------------------------------------------------------------}
{ TConnectActiveResp                                                }
{ ------------------------------------------------------------------}
constructor TConnectActiveResp.Create(PLCI : dword);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := CONNECT_ACTIVE_RESP;
  end;

  AddData(@PLCI, SizeOf(PLCI));
end;
{ -------------------------------------------------------------------}
{ TConnectB3Req                                                      }
{ ------------------------------------------------------------------ }
constructor TConnectB3Req.Create(PLCI : dword;
                                 NCPI : TStruct);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord) +
                NCPI.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := CONNECT_B3_REQ;
  end;

  AddData(@PLCI, SizeOf(PLCI));
  AddData(NCPI.Header, NCPI.Length);

  NCPI.Free;
end;
{  ------------------------------------------------------------------}
{ TConnectB3Ind                                                      }
{ ------------------------------------------------------------------ }
constructor TConnectB3Ind.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@NCCI, SizeOf(NCCI));
  NCPI := TStruct.CreateFromData(FDataPtr);
end;
destructor TConnectB3Ind.destroy;
begin
  NCPI.Free;
  inherited destroy;
end;
{  ------------------------------------------------------------------}
{ TConnectB3Resp                                                     }
{ ------------------------------------------------------------------ }
constructor TConnectB3Resp.Create(NCCI   : dword;
                                  Reject : word;
                                  NCPI   : TStruct);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord) +
                Sizeof(word) + 
                NCPI.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := CONNECT_B3_RESP;
  end;

  AddData(@NCCI, SizeOf(NCCI));
  AddData(@Reject, SizeOf(Reject));
  AddData(NCPI.Header, NCPI.Length);
  NCPI.Free;
end;
{  ------------------------------------------------------------------}
{ TConnectB3ActiveInd                                                }
{ ------------------------------------------------------------------ }
constructor TConnectB3ActiveInd.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@NCCI, SizeOf(NCCI));
  NCPI := TStruct.CreateFromData(FDataPtr);
end;
destructor TConnectB3ActiveInd.destroy;
begin
  NCPI.Free;
  inherited destroy;
end;
{  ------------------------------------------------------------------}
{ TConnectB3ActiveResp                                               }
{ ------------------------------------------------------------------ }
constructor TConnectB3ActiveResp.Create(NCCI   : dword);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := CONNECT_B3_ACTIVE_RESP;
  end;

  AddData(@NCCI, SizeOf(NCCI));
end;
{  ------------------------------------------------------------------}
{ TConnectB3T90ActiveInd                                             }
{ ------------------------------------------------------------------ }
constructor TConnectB3T90ActiveInd.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@NCCI, SizeOf(NCCI));
  NCPI := TStruct.CreateFromData(FDataPtr);
end;
destructor TConnectB3T90ActiveInd.destroy;
begin
  NCPI.Free;
  inherited destroy;
end;
{  ------------------------------------------------------------------}
{ TConnectB3T90ActiveResp                                            }
{ ------------------------------------------------------------------ }
constructor TConnectB3T90ActiveResp.Create(NCCI   : dword);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := CONNECT_B3_T90_ACTIVE_RESP;
  end;

  AddData(@NCCI, SizeOf(NCCI));
end;
{ -----------------------------------------------------------------  }
{ TDataB3Req                                                         }
{ -----------------------------------------------------------------  }
constructor TDataB3Req.Create(NCCI : dword;
                              Data : dword;
                              DataLength : word;
                              DataHandle : word;
                              Flags : word);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                2 * SizeOf(dWord) +
                3 * SizeOf(word);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := DATA_B3_REQ;
  end;

  AddData(@NCCI, SizeOf(NCCI));
  AddData(@Data, SizeOf(Data));
  AddData(@DataLength, SizeOf(DataLength));
  AddData(@DataHandle, SizeOf(DataHandle));
  AddData(@Flags, SizeOf(Flags));
end;
{ -----------------------------------------------------------------  }
{ TDataB3Conf                                                        }
{ -----------------------------------------------------------------  }
constructor TDataB3Conf.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@NCCI, SizeOf(NCCI));
  GetData(@DataHandle, SizeOf(DataHandle));
  GetData(@Info, SizeOf(Info));
end;
{ -----------------------------------------------------------------  }
{ TDataB3Ind                                                         }
{ -----------------------------------------------------------------  }
constructor TDataB3Ind.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@NCCI, SizeOf(NCCI));
  GetData(@Data, SizeOf(Data));
  GetData(@DataLength, SizeOf(DataLength));
  GetData(@DataHandle, SizeOf(DataHandle));
  GetData(@Flags, SizeOf(Flags));
end;
{ -----------------------------------------------------------------  }
{ TDataB3Resp                                                        }
{ -----------------------------------------------------------------  }
constructor TDataB3Resp.Create(NCCI   : dword;
                               DataHandle : word);

var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord) +
                SizeOf(word);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := DATA_B3_RESP;
  end;

  AddData(@NCCI, SizeOf(NCCI));
  AddData(@DataHandle, SizeOf(DataHandle));
end;
{ -------------------------------------------------------------------}
{ TDisconnectB3Req                                                   }
{ ------------------------------------------------------------------ }
constructor TDisconnectB3Req.Create(NCCI : dword;
                                    NCPI : TStruct);

var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord) +
                NCPI.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := DISCONNECT_B3_REQ;
  end;

  AddData(@NCCI, SizeOf(NCCI));
  AddData(NCPI.Header, NCPI.Length);

  NCPI.Free;
end;
{  ------------------------------------------------------------------}
{ TDisconnectB3Ind                                                   }
{ ------------------------------------------------------------------ }
constructor TDisConnectB3Ind.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@NCCI, SizeOf(NCCI));
  GetData(@Reason_B3, SizeOf(Reason_B3));
  NCPI := TStruct.CreateFromData(FDataPtr);
end;
destructor TDisConnectB3Ind.destroy;
begin
  NCPI.Free;
  inherited destroy;
end;
{  ------------------------------------------------------------------}
{ TDisconnectB3Resp                                                  }
{ ------------------------------------------------------------------ }
constructor TDisconnectB3Resp.Create(NCCI   : dword);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := DISCONNECT_B3_RESP;
  end;

  AddData(@NCCI, SizeOf(NCCI));
end;
{  ------------------------------------------------------------------}
{ TDisconnectInd                                                     }
{ ------------------------------------------------------------------ }
constructor TDisconnectInd.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@PLCI, SizeOf(PLCI));
  GetData(@Reason, SizeOf(Reason));
end;
{  ------------------------------------------------------------------}
{ TDisconnectReq                                                     }
{ ------------------------------------------------------------------ }
constructor TDisconnectReq. Create(PLCI : dWord;
                                   AdditionalInfo : TStruct);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord) +
                AdditionalInfo.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := DISCONNECT_REQ;
  end;

  AddData(@PLCI, SizeOf(PLCI));
  AddData(AdditionalInfo.Header, AdditionalInfo.Length);
  AdditionalInfo.Free;
end;
{  ------------------------------------------------------------------}
{ TDisconnectResp                                                    }
{ ------------------------------------------------------------------ }
constructor TDisconnectResp. Create(PLCI : dWord);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
               SizeOf(dWord);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := DISCONNECT_RESP;
  end;

  AddData(@PLCI, SizeOf(PLCI));
end;
{  ------------------------------------------------------------------}
{ TInfoReq                                                           }
{ ------------------------------------------------------------------ }
constructor TInfoReq.Create(PLCI : dword;
                            CalledPartyNumber : TCalledPartyNumber;
                            AdditionalInfo : TStruct);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
               SizeOf(dWord) +
               CalledPartyNumber.Length +
               AdditionalInfo.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := INFO_REQ;
  end;

  AddData(@PLCI, SizeOf(PLCI));
  AddData(CalledPartyNumber.Header, CalledPartyNumber.Length);
  AddData(AdditionalInfo.Header, AdditionalInfo.Length);
  CalledPartyNumber.Free;
  AdditionalInfo.Free;
end;
{ -------------------------------------------------------------------}
{ TInfoInd
  ------------------------------------------------------------------
  Info number (1)
    Bit 15-14:   1x..           00..            01..
    Bit 13- 8:     00 0000        00 0000         00 0000
    Bit  7- 0:   message type   info element    supplementary info type
                 (2)            identifier        00 0000 charges (1)
                                (3)               00 0001 charges in nat. currency

  Info element:  empty struct   info            charges as dword
  (5)                           element (4)
  ------------------------------------------------------------------
  (1) see Capi, Parameter Desciption, info number
  (2) see ETS 300102-1 4.4, tab. 4.2 message types
  (3) see ETS 300102-1, 4.5, table 4.3
  (4) see ETS 300102-1, 4.5ff, 4.6
  (5) see Capi, Parameter Desciption, info element
  see also Capi, Parameter Desciption, Info Mask !!!  
  ------------------------------------------------------------------
  Example: Info Number = 0x28 (= 0000 0000 0010 1000)
  - Bit 15/14 = 00 so Bit 7-0 = info element identifier (= 0x28)
  - ETS 300102-1 table 4.3: info element (0x28) = Display
  - ETS 300102-1, 4.5.15 Display describes the struct display
{ ------------------------------------------------------------------ }
constructor TInfoInd.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@PLCI, SizeOf(PLCI));
  GetData(@InfoNumber, SizeOf(InfoNumber));
  InfoElement := TStruct.CreateFromData(FDataPtr);
end;
destructor TInfoInd.Destroy;
begin
  InfoElement.Free;
  inherited Destroy;
end;
{  ------------------------------------------------------------------}
{ TInfoResp                                                          }
{ ------------------------------------------------------------------ }
constructor TInfoResp.Create(PLCI : dWord);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
                SizeOf(dWord);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := INFO_RESP;
  end;

  AddData(@PLCI, SizeOf(PLCI));
end;
{ ------------------------------------------------------------------}
{ TAlertReq                                                         }
{ ------------------------------------------------------------------}
constructor TAlertReq.Create(PLCI : dword; AdditionalInfo : TStruct);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
               SizeOf(dWord) +
               AdditionalInfo.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := ALERT_REQ;
  end;

  AddData(@PLCI, SizeOf(PLCI));
  AddData(AdditionalInfo.Header, AdditionalInfo.Length);
  AdditionalInfo.Free;
end;
{ -------------------------------------------------------------------}
{ TFacilityReq                                                       }
{ -------------------------------------------------------------------}
{ sample: sending DTMF-Digits                                        }
{    TFacilityReq.Create(                                            }
{       1,                                        { Controller       }
{       fsDTMF,                                   { FacilitySelector }
{       TDTMFFacilityReqParameter.Create(         { ReqParameter     }
{         cDTMFSendDigits,                        { Command          }
{         cDTMFDefToneDuration,                   { ToneDuration     }
{         cDTMFDefGapDuration,                    { GapDuration      }
{         TDTMFDigits.Create('12345')));          { DTMFDigits       }
{ -------------------------------------------------------------------}
constructor TFacilityReq.Create(Controller_PLCI_NCCI : dWord;
                                FacilitySelector : word;
                                FacilityParameters : TStruct);
var MsgLength: word;
begin
  inherited Create;
  MsgLength := Header.Length +
               SizeOf(dWord) + SizeOf(Word) +
               FacilityParameters.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := FACILITY_REQ;
  end;
  AddData(@Controller_PLCI_NCCI, SizeOf(Controller_PLCI_NCCI));
  AddData(@FacilitySelector, SizeOf(FacilitySelector));
  AddData(FacilityParameters.Header, FacilityParameters.Length);
  FacilityParameters.Free;
end;
{ -------------------------------------------------------------------}
{ TFacilityConf
{ -------------------------------------------------------------------}
{ sample: confirmation of sent DTMF-Digits
    with TFacilityConf(aCapiMessage) do begin
      if FacilitySelector = fsDTMF then begin
        fcp := TDTMFFacilityConfParameter.
                            CreateFromData(FacilityParameters.Header);
        OK := (fcp.Information = cDTMFInfoSuccess);
        fcp.Free;
      end;
    end;                                                             }
{ -------------------------------------------------------------------}
constructor TFacilityConf.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@Controller_PLCI_NCCI, SizeOf(Controller_PLCI_NCCI));
  GetData(@Info, SizeOf(Info));
  GetData(@FacilitySelector, SizeOf(FacilitySelector));
  FacilityParameters := TStruct.CreateFromData(FDataPtr);
  inc(FDataPtr, FacilityParameters.Length);
end;
destructor  TFacilityConf.destroy;
begin
  FacilityParameters.Free;
  inherited destroy;
end;
{ -------------------------------------------------------------------}
{ TFacilityInd                                                       }
{ -------------------------------------------------------------------}
{ sample: receiving DTMF digits
    with TFacilityInd(aCapiMessage) do begin
      if FacilitySelector = fsDTMF then begin
        fcp := TDTMFFacilityIndParameter.
                            CreateFromData(FacilityParameters.Header);
        DTMFDigits := fcp.Digits;
        fcp.Free;
      end;
    end;                                                             }
{ -------------------------------------------------------------------}
constructor TFacilityInd.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@Controller_PLCI_NCCI, SizeOf(Controller_PLCI_NCCI));
  GetData(@FacilitySelector, SizeOf(FacilitySelector));
  FacilityParameters := TStruct.CreateFromData(FDataPtr);
  inc(FDataPtr, FacilityParameters.Length);
end;
destructor  TFacilityInd.destroy;
begin
  FacilityParameters.Free;
  inherited destroy;
end;
{ -------------------------------------------------------------------}
{ TFacilityResp                                                      }
{ -------------------------------------------------------------------}
constructor TFacilityResp.Create(Controller_PLCI_NCCI : dWord;
                                 FacilitySelector : word;
                                 FacilityParameters : TStruct);
var MsgLength: word;
begin
  inherited Create;
  MsgLength := Header.Length +
               SizeOf(dWord) + SizeOf(Word) +
               FacilityParameters.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := FACILITY_RESP;
  end;
  AddData(@Controller_PLCI_NCCI, SizeOf(Controller_PLCI_NCCI));
  AddData(@FacilitySelector, SizeOf(FacilitySelector));
  AddData(FacilityParameters.Header, FacilityParameters.Length);
  FacilityParameters.Free;
end;
{ -------------------------------------------------------------------}
{ TResetB3Req                                                        }
{ ------------------------------------------------------------------ }
constructor TResetB3Req.Create(NCCI : dword;
                               NCPI : TStruct);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
               SizeOf(dWord) +
               NCPI.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := RESET_B3_REQ;
  end;

  AddData(@NCCI, SizeOf(NCCI));
  AddData(NCPI.Header, NCPI.Length);

  NCPI.Free;
end;
{  ------------------------------------------------------------------}
{ TResetB3Ind                                                        }
{ ------------------------------------------------------------------ }
constructor TResetB3Ind.CreateFromData(aDataPtr : pointer);
begin
  inherited CreateFromData(aDataPtr);
  GetData(@NCCI, SizeOf(NCCI));
  NCPI := TStruct.CreateFromData(FDataPtr);
end;
destructor TResetB3Ind.destroy;
begin
  NCPI.Free;
  inherited destroy;
end;
{  ------------------------------------------------------------------}
{ TResetB3Resp                                                       }
{ ------------------------------------------------------------------ }
constructor TResetB3Resp.Create(NCCI : dword);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
               Sizeof(dword);
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := RESET_B3_RESP;
  end;

  AddData(@NCCI, SizeOf(NCCI));
end;
{ -------------------------------------------------------------------}
{ TSelectBProtocolReq                                                }
{ ------------------------------------------------------------------ }
constructor TSelectBProtocolReq.Create(PLCI : dword;
                                       BProtocol : TBProtocol);
var MsgLength : word;
begin
  inherited Create;
  MsgLength := Header.Length +
               SizeOf(dWord) +
               BProtocol.Length;
  MakeData(MsgLength);

  with Header do begin
    TotalLength := MsgLength;
    WordCommand := SELECT_B_PROTOCOL_REQ;
  end;

  AddData(@PLCI, SizeOf(PLCI));
  AddData(BProtocol.Header, BProtocol.Length);

  BProtocol.Free;
end;

{ -------------------------------------------------------------------}
{  CapiCreateMessage                                                 }
{ -------------------------------------------------------------------}
function CapiCreateMessage(aCMsg : pointer) : TCapiMessage;
var MsgHeader   : TCapiMessageHeader;
begin
  { MsgHeader is used only temporarily to find out what kind of
    message is received }
  MsgHeader := TCapiMessageHeader.CreateFromData(aCMsg);
  case MsgHeader.WordCommand of
    DATA_B3_CONF       : CapiCreateMessage := TDataB3Conf.CreateFromData(aCMsg);
    DATA_B3_IND        : CapiCreateMessage := TDataB3Ind.CreateFromData(aCMsg);
    ALERT_CONF         : CapiCreateMessage := TAlertConf.CreateFromData(aCMsg);
    LISTEN_CONF        : CapiCreateMessage := TListenConf.CreateFromData(aCMsg);
    CONNECT_IND        : CapiCreateMessage := TConnectInd.CreateFromData(aCMsg);
    CONNECT_CONF       : CapiCreateMessage := TConnectConf.CreateFromData(aCMsg);
    CONNECT_ACTIVE_IND : CapiCreateMessage := TConnectActiveInd.CreateFromData(aCMsg);
    CONNECT_B3_CONF    : CapiCreateMessage := TConnectB3Conf.CreateFromData(aCMsg);
    CONNECT_B3_IND     : CapiCreateMessage := TConnectB3Ind.CreateFromData(aCMsg);
    CONNECT_B3_ACTIVE_IND     : CapiCreateMessage := TConnectB3ActiveInd.CreateFromData(aCMsg);
    CONNECT_B3_T90_ACTIVE_IND : CapiCreateMessage := TConnectB3T90ActiveInd.CreateFromData(aCMsg);
    DISCONNECT_CONF    : CapiCreateMessage := TDisconnectConf.CreateFromData(aCMsg);
    DISCONNECT_B3_IND  : CapiCreateMessage := TDisconnectB3Ind.CreateFromData(aCMsg);
    DISCONNECT_B3_CONF : CapiCreateMessage := TDisconnectB3Conf.CreateFromData(aCMsg);
    DISCONNECT_IND     : CapiCreateMessage  := TDisconnectInd.CreateFromData(aCMsg);
    INFO_CONF          : CapiCreateMessage := TInfoConf.CreateFromData(aCMsg);
    INFO_IND           : CapiCreateMessage := TInfoInd.CreateFromData(aCMsg);
    FACILITY_IND       : CapiCreateMessage := TFacilityInd.CreateFromData(aCMsg);
    FACILITY_CONF      : CapiCreateMessage := TFacilityConf.CreateFromData(aCMsg);
    RESET_B3_IND       : CapiCreateMessage := TResetB3Ind.CreateFromData(aCMsg);
    RESET_B3_CONF      : CapiCreateMessage := TResetB3Conf.CreateFromData(aCMsg);
    SELECT_B_PROTOCOL_CONF : CapiCreateMessage := TSelectBProtocolConf.CreateFromData(aCMsg);
  else
    CapiCreateMessage := TCapiMessage.CreateFromData(aCMsg);
  end;
  MsgHeader.Free;                 { release temporariliy used header }
end;
{ -------------------------------------------------------------------}
{  CapiFreeMessage                                                   }
{  Used to release CapiMessages                                      }
{ -------------------------------------------------------------------}
procedure CapiFreeMessage(var aCapiMessage : TCapiMessage);
begin
  aCapiMessage.Free;
  aCapiMessage := nil;
end;
end.
